package utep.plmr;

import javax.swing.*;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import java.io.*;
import java.awt.*;
import java.awt.event.ActionEvent;

import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.CategoryAxis;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;

/**
 * Dialog that displays and sets all options for a Chart
 * @author derDoc
 *
 */
public class ChartPrefs implements Serializable {

	private static final long serialVersionUID = 310719801L;

	/** the dialog main window */
	transient public JDialog theDia;

	/** background img for the chart */
	transient public File BackgroundImg = null;

	/** its name */
	public String BackgroundImgName = null;

	public PlotOrientation PlotOr = PlotOrientation.HORIZONTAL;

	/** title of the chart */
	public String Title;

	// X
	/**min val of axis */ 
	public double X_Min;

	/**min val changed */
	public boolean X_Min_changed = false;

	/**max val */
	public double X_Max = 1;

	public boolean X_Max_changed = false;

	/**currently selected min val before starting of dia */
	public double X_Min_Curr;

	public double X_Max_Curr = 1;

	/**caption */
	public String X_Capt;

	// Y
	public double Y_Min;

	public boolean Y_Min_changed = false;

	public double Y_Max = 1;

	public boolean Y_Max_changed = false;

	public double Y_Min_Curr;

	public double Y_Max_Curr = 1;

	public String Y_Capt;

	/**
	 * Starts the ChartPrefs Dialog
	 *
	 */
	public ChartPrefs() {
		theDia = new JDialog((JFrame) null, "Chart Preferences");
	}

	/**
	 * Creates all fields in the dialog
	 *
	 */
	public void initialize() {
		if (theDia == null)
			theDia = new JDialog((JFrame) null, "Chart Preferences");

		X_Min_changed = false;
		X_Max_changed = false;
		Y_Min_changed = false;
		Y_Max_changed = false;

		// top
		JPanel jptop = new JPanel();
		jptop.add(new JLabel("Plot Orientation"));
		final JComboBox jcc = new JComboBox(new String[] { "Horizontal",
				"Vertical" });
		jcc.setSelectedIndex((PlotOr == PlotOrientation.HORIZONTAL) ? 0 : 1);
		jcc.addActionListener(new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				if (jcc.getSelectedIndex() == 0)
					PlotOr = PlotOrientation.HORIZONTAL;
				else
					PlotOr = PlotOrientation.VERTICAL;
			}
		});
		jptop.add(jcc);
		theDia.getContentPane().add(jptop, BorderLayout.NORTH);

		// center
		JPanel jp = new JPanel(new GridLayout(3, 4));
		// Col Headers (Row1)
		jp.add(new JLabel(""));
		jp.add(new JLabel("Min"));
		jp.add(new JLabel("Max"));
		jp.add(new JLabel("Caption"));
		// Row2
		jp.add(new JLabel("X:"));
		final JTextField jt_xmin = new JTextField(new Double(X_Min).toString(),
				5);
		if (X_Min == Double.NaN)
			jt_xmin.setEditable(false);

		jt_xmin.getDocument().addDocumentListener(new DocumentListener() {
			public void insertUpdate(DocumentEvent e) {
				Upd();
			}

			public void removeUpdate(DocumentEvent e) {
				Upd();
			}

			public void changedUpdate(DocumentEvent e) {
				Upd();
			}

			private void Upd() {
				if (jt_xmin.getText().length() > 0) {
					if (jt_xmin.getText().length() == 1
							&& jt_xmin.getText().equals("-"))
						return;
					try {
						X_Min = new Double(jt_xmin.getText()).doubleValue();
						X_Min_changed = true;

					} catch (Exception e) {
						JOptionPane.showMessageDialog(null,
								"Please enter a real value!");
					}
				}
			}
		});
		jp.add(jt_xmin);

		final JTextField jt_xmax = new JTextField(new Double(X_Max).toString(),
				5);
		if (X_Max == Double.NaN)
			jt_xmax.setEditable(false);

		jt_xmax.getDocument().addDocumentListener(new DocumentListener() {
			public void insertUpdate(DocumentEvent e) {
				Upd();
			}

			public void removeUpdate(DocumentEvent e) {
				Upd();
			}

			public void changedUpdate(DocumentEvent e) {
				Upd();
			}

			private void Upd() {
				if (jt_xmax.getText().length() > 0) {
					if (jt_xmax.getText().length() == 1
							&& jt_xmax.getText().equals("-"))
						return;
					try {
						X_Max = new Double(jt_xmax.getText()).doubleValue();
						X_Max_changed = true;
					} catch (Exception e) {
						JOptionPane.showMessageDialog(null,
								"Please enter a real value!");
					}
				}
			}
		});
		jp.add(jt_xmax);

		final JTextField jt_xcapt = new JTextField(X_Capt, 10);
		jt_xcapt.getDocument().addDocumentListener(new DocumentListener() {
			public void insertUpdate(DocumentEvent e) {
				Upd();
			}

			public void removeUpdate(DocumentEvent e) {
				Upd();
			}

			public void changedUpdate(DocumentEvent e) {
				Upd();
			}

			private void Upd() {
				X_Capt = jt_xcapt.getText();
			}
		});
		jp.add(jt_xcapt);

		// Row3
		jp.add(new JLabel("Y:"));
		final JTextField jt_ymin = new JTextField(new Double(Y_Min).toString(),
				5);
		jt_ymin.getDocument().addDocumentListener(new DocumentListener() {
			public void insertUpdate(DocumentEvent e) {
				Upd();
			}

			public void removeUpdate(DocumentEvent e) {
				Upd();
			}

			public void changedUpdate(DocumentEvent e) {
				Upd();
			}

			private void Upd() {
				if (jt_ymin.getText().length() > 0) {
					if (jt_ymin.getText().length() == 1
							&& jt_ymin.getText().equals("-"))
						return;
					try {
						Y_Min = new Double(jt_ymin.getText()).doubleValue();
						Y_Min_changed = true;
					} catch (Exception e) {
						JOptionPane.showMessageDialog(null,
								"Please enter a real value!");
					}
				}
			}
		});
		jp.add(jt_ymin);

		final JTextField jt_ymax = new JTextField(new Double(Y_Max).toString(),
				5);
		jt_ymax.getDocument().addDocumentListener(new DocumentListener() {
			public void insertUpdate(DocumentEvent e) {
				Upd();
			}

			public void removeUpdate(DocumentEvent e) {
				Upd();
			}

			public void changedUpdate(DocumentEvent e) {
				Upd();
			}

			private void Upd() {
				if (jt_ymax.getText().length() > 0) {
					if (jt_ymax.getText().length() == 1
							&& jt_ymax.getText().equals("-"))
						return;
					try {
						Y_Max = new Double(jt_ymax.getText()).doubleValue();
						Y_Max_changed = true;
					} catch (Exception e) {
						JOptionPane.showMessageDialog(null,
								"Please enter a real value!");
					}
				}
			}
		});
		jp.add(jt_ymax);

		final JTextField jt_ycapt = new JTextField(Y_Capt, 10);
		jt_ycapt.getDocument().addDocumentListener(new DocumentListener() {
			public void insertUpdate(DocumentEvent e) {
				Upd();
			}

			public void removeUpdate(DocumentEvent e) {
				Upd();
			}

			public void changedUpdate(DocumentEvent e) {
				Upd();
			}

			private void Upd() {
				Y_Capt = jt_ycapt.getText();
			}
		});
		jp.add(jt_ycapt);

		theDia.getContentPane().add(jp, BorderLayout.CENTER);

		// bottom
		JPanel jp_bottom = new JPanel();
		JButton jb = new JButton("Background Image");
		jb.addActionListener(new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				JFileChooser jfc = new JFileChooser(System
						.getProperty("user.dir"));
				jfc.setFileSelectionMode(JFileChooser.FILES_ONLY);
				jfc.setMultiSelectionEnabled(false);
				if (BackgroundImgName != null)
					jfc.setSelectedFile(new File(BackgroundImgName));
				if (JFileChooser.APPROVE_OPTION == jfc.showOpenDialog(null)) {
					BackgroundImg = jfc.getSelectedFile();
					BackgroundImgName = jfc.getSelectedFile().getAbsolutePath();
				}
			}
		});
		jp_bottom.add(new JLabel("Title:"));
		final JTextField jt_tit = new JTextField(Title, 15);
		jt_tit.getDocument().addDocumentListener(new DocumentListener() {
			public void insertUpdate(DocumentEvent e) {
				Upd();
			}

			public void removeUpdate(DocumentEvent e) {
				Upd();
			}

			public void changedUpdate(DocumentEvent e) {
				Upd();
			}

			private void Upd() {
				Title = jt_tit.getText();
			}
		});
		jp_bottom.add(jt_tit);
		jp_bottom.add(jb);
		theDia.getContentPane().add(jp_bottom, BorderLayout.SOUTH);
	}
	

	/**
	 * Initializes all fields in this class from the Chart and shows it
	 * @param show
	 * 				true - actually show the dialog as a window
	 */
	public static void show_prefs(boolean show) {
		if (pwhn.cP.Res_Method > 0 && pwhn.cP.Res_Method < 10) {
			JFreeChart chart = MainWindow.Res_chart;

			ChartPrefs cp = pwhn.cP.chart_pref;
			cp.Title = chart.getTitle().getText();

			if (pwhn.cP.Res_Method < 7) { // XYPlots
				XYPlot xy = (XYPlot) chart.getPlot();
				cp.PlotOr = xy.getOrientation();

				ValueAxis va = xy.getDomainAxis();
				cp.X_Min_Curr = va.getLowerBound();
				if (cp.X_Min_Curr < cp.X_Min)
					cp.X_Min = cp.X_Min_Curr;
				cp.X_Max_Curr = va.getUpperBound();
				if (cp.X_Max_Curr > cp.X_Max)
					cp.X_Max = cp.X_Max_Curr;
				cp.X_Capt = va.getLabel();

				va = xy.getRangeAxis();
				cp.Y_Min_Curr = va.getLowerBound();
				if (cp.Y_Min_Curr < cp.Y_Min)
					cp.Y_Min = cp.Y_Min_Curr;
				cp.Y_Max_Curr = va.getUpperBound();
				if (cp.Y_Max_Curr > cp.Y_Max)
					cp.Y_Max = cp.Y_Max_Curr;
				cp.Y_Capt = va.getLabel();

			} else { // CatPlots
				CategoryPlot xy = (CategoryPlot) chart.getPlot();
				cp.PlotOr = xy.getOrientation();

				CategoryAxis ca = xy.getDomainAxis();
				cp.X_Min_Curr = Double.NaN;
				cp.X_Max_Curr = Double.NaN;
				cp.X_Capt = ca.getLabel();

				ValueAxis va = xy.getRangeAxis();
				cp.Y_Min_Curr = va.getLowerBound();
				cp.Y_Max_Curr = va.getUpperBound();
				cp.Y_Capt = va.getLabel();
			}
			if (show) {
				cp.initialize();
				cp.theDia.pack();
				cp.theDia.setModal(true);
				cp.theDia.setVisible(true);
				UpdatePrefs(cp);
			}
		}
	}

	/**
	 * Updates all the fields of the Chart from this Dialog instance's fields
	 * repaints the Chart afterwards
	 * @param cp
	 * 			a ref to this class, since this function is static
	 */
	public static void UpdatePrefs(ChartPrefs cp) {
		if (pwhn.cP.Res_Method > 0 && pwhn.cP.Res_Method < 10) {
			JFreeChart chart = MainWindow.Res_chart;

			// and update stuff
			chart.getTitle().setText(cp.Title);
			if (cp.BackgroundImg != null) {
				Image i = Toolkit.getDefaultToolkit().getImage(
						cp.BackgroundImg.getAbsolutePath());
				chart.getPlot().setBackgroundImage(i);
			} else if (cp.BackgroundImgName != null) {
				try {
					Image i = Toolkit.getDefaultToolkit().getImage(
							cp.BackgroundImgName);
					chart.getPlot().setBackgroundImage(i);
				} catch (Exception e) {}
			}
			if (pwhn.cP.Res_Method < 7) {
				XYPlot xy = (XYPlot) chart.getPlot();
				xy.setOrientation(cp.PlotOr);

				ValueAxis va = xy.getDomainAxis();
				if (cp.X_Min_changed)
					va.setLowerBound(cp.X_Min);
				if (cp.X_Max_changed)
					va.setUpperBound(cp.X_Max);
				va.setLabel(cp.X_Capt);

				va = xy.getRangeAxis();
				if (cp.Y_Min_changed)
					va.setLowerBound(cp.Y_Min);
				if (cp.Y_Max_changed)
					va.setUpperBound(cp.Y_Max);
				va.setLabel(cp.Y_Capt);

			} else {
				CategoryPlot xy = (CategoryPlot) chart.getPlot();
				xy.setOrientation(cp.PlotOr);

				CategoryAxis ca = xy.getDomainAxis();
				ca.setLabel(cp.X_Capt);

				ValueAxis va = xy.getRangeAxis();
				if (cp.Y_Min_changed)
					va.setLowerBound(cp.Y_Min);
				if (cp.Y_Max_changed)
					va.setUpperBound(cp.Y_Max);
				va.setLabel(cp.Y_Capt);
			}
			MainWindow.myFrame.repaint();
			MainWindow.Res_chartPanel.repaint();
		}
	}


}
