package utep.plmr;

import java.util.Enumeration;

import javax.swing.*;

import org.jfree.chart.*;
import org.jfree.chart.labels.StandardXYZToolTipGenerator;
import org.jfree.chart.plot.*;
import org.jfree.chart.renderer.xy.*;
import org.jfree.chart.renderer.category.*;
import org.jfree.chart.urls.*;
import org.jfree.data.category.*;
import org.jfree.data.general.*;
import org.jfree.data.xy.*;
import org.jfree.chart.axis.*;

import org.jgraph.*;
import org.jgraph.graph.*;

import utep.plmr.Visitors.DelayHistoVisitor;
import utep.plmr.jfree.XYZSeries;
import utep.plmr.jfree.XYZSeriesCollection;
import utep.plmr.jfree.myLASRenderer;
import utep.plmr.jfree.myXYPlot;

/**
 * Helper-Class for creating and updating charts and graphs, used by MainWindow 
 * @author derDoc
 *
 */
public class Charting {

	public static final String		DEF_CHART_TITLE			= "PWHN Query Results";

	public static final String		DEF_DOM_TXT				= "Domain";

	public static final String		DEF_RNG_TXT				= "Range";
	
	/**
	 * creates a chart
	 * @param iType
	 * 				Type to create, as given in the type combobox
	 * @param Res_dataSet
	 * 				the dataset of the graph from the mainwindo
	 * @param Res_chart
	 * 				the Chart that sits in the MainWindow
	 * @param Res_chartPanel
	 * 				The ChartPanel that contains the Chart
	 */
	public static Object[] createChart(int iType, Dataset Res_dataSet, JFreeChart Res_chart,
			ChartPanel Res_chartPanel) {
		
		JFreeChart chart = null;
		Res_dataSet = null;

		Object[] ret_arr = new Object[2];
		
		
		switch (iType) {

			case 1: // Bar
				Res_dataSet = new XYSeriesCollection();
				chart = ChartFactory.createXYBarChart(DEF_CHART_TITLE, // title
						DEF_DOM_TXT, // domain axis label
						false,
						DEF_RNG_TXT, // range axis label
						new XYBarDataset((XYDataset) Res_dataSet, 0.5), // data
						PlotOrientation.VERTICAL, // orientation
						true, // include legend
						true, // tooltips?
						false // URLs?
						);

				break;

			case 2: // Line
				Res_dataSet = new XYSeriesCollection();

				chart = ChartFactory.createXYLineChart(DEF_CHART_TITLE, // title
						DEF_DOM_TXT, // domain axis label
						DEF_RNG_TXT, // range axis label
						new XYBarDataset((XYDataset) Res_dataSet, 1), // data
						PlotOrientation.VERTICAL, // orientation
						true, // include legend
						true, // tooltips?
						false // URLs?
						);

				break;
			case 3: // Dots
				Res_dataSet = new XYSeriesCollection();

				chart = ChartFactory.createScatterPlot(DEF_CHART_TITLE, // title
						DEF_DOM_TXT, // domain axis label
						DEF_RNG_TXT, // range axis label
						new XYBarDataset((XYDataset) Res_dataSet, 1), // data
						PlotOrientation.VERTICAL, // orientation
						true, // include legend
						true, // tooltips?
						false // URLs?
						);

				break;

			case 4: // Area
				Res_dataSet = new XYSeriesCollection();

				chart = ChartFactory.createXYAreaChart(DEF_CHART_TITLE, // title
						DEF_DOM_TXT, // domain axis label
						DEF_RNG_TXT, // range axis label
						new XYBarDataset((XYDataset) Res_dataSet, 1), // data
						PlotOrientation.VERTICAL, // orientation
						true, // include legend
						true, // tooltips?
						false // URLs?
						);

				break;

			case 5: // XYZ Bubble
				Res_dataSet = new XYZSeriesCollection();

				chart = ChartFactory.createBubbleChart(DEF_CHART_TITLE, // title
						DEF_DOM_TXT, // domain axis label
						DEF_RNG_TXT, // range axis label
						(XYZDataset) Res_dataSet, // data
						PlotOrientation.VERTICAL, // orientation
						true, // include legend
						true, // tooltips?
						false // URLs?
						);

				myXYPlot myPlot = new myXYPlot((XYDataset) Res_dataSet, chart
						.getXYPlot().getDomainAxis(), chart.getXYPlot()
						.getRangeAxis(), chart.getXYPlot().getRenderer());

				myPlot.setOrientation(PlotOrientation.VERTICAL);

				JFreeChart chart2 = new JFreeChart(
						chart.getTitle().getText(),
						JFreeChart.DEFAULT_TITLE_FONT,
						myPlot,
						true);
				chart = chart2;

				break;
			case 6: // XYZ Color
				Res_dataSet = new XYZSeriesCollection();

				chart = ChartFactory.createScatterPlot(DEF_CHART_TITLE, // title
						DEF_DOM_TXT, // domain axis label
						DEF_RNG_TXT, // range axis label
						(XYZDataset) Res_dataSet, // data
						PlotOrientation.VERTICAL, // orientation
						true, // include legend
						true, // tooltips?
						false // URLs?
						);

				XYItemRenderer renderer = new myLASRenderer(false, true);
				renderer
						.setBaseToolTipGenerator(new StandardXYZToolTipGenerator());
				renderer.setURLGenerator(new StandardXYZURLGenerator());

				myXYPlot p = new myXYPlot((XYDataset) Res_dataSet, chart
						.getXYPlot().getDomainAxis(), chart.getXYPlot()
						.getRangeAxis(), renderer);
				p.setOrientation(PlotOrientation.VERTICAL);

				JFreeChart c = new JFreeChart(
						chart.getTitle().getText(),
						JFreeChart.DEFAULT_TITLE_FONT,
						p,
						true);
				chart = c;

				break;

			case 7: // Category Bar
				Res_dataSet = new DefaultCategoryDataset();
				chart = ChartFactory.createBarChart(DEF_CHART_TITLE, // chart
						// title
						DEF_DOM_TXT, // domain axis label
						DEF_RNG_TXT, // range axis label
						(CategoryDataset) Res_dataSet, // data
						PlotOrientation.VERTICAL, // orientation
						true, // include legend
						true, // tooltips?
						false // URLs?
						);
				break;
			case 8: // Category Line
				Res_dataSet = new DefaultCategoryDataset();
				chart = ChartFactory.createLineChart(DEF_CHART_TITLE, // chart
						DEF_DOM_TXT, // domain axis label
						DEF_RNG_TXT, // range axis label
						(CategoryDataset) Res_dataSet, // data
						PlotOrientation.VERTICAL, // orientation
						true, // include legend
						true, // tooltips?
						false // URLs?
						);
				break;
			case 9: // Category Area
				Res_dataSet = new DefaultCategoryDataset();
				chart = ChartFactory.createAreaChart(DEF_CHART_TITLE, // chart
						DEF_DOM_TXT, // domain axis label
						DEF_RNG_TXT, // range axis label
						(CategoryDataset) Res_dataSet, // data
						PlotOrientation.VERTICAL, // orientation
						true, // include legend
						true, // tooltips?
						false // URLs?
						);
				break;

			case 11:
				Res_dataSet = new XYSeriesCollection();

				chart = ChartFactory.createXYLineChart(
						"RTT vs. payloads histogram", // title
						"Link delay / ms", // range axis label
						"Payload / byte", // domain axis label
						new XYBarDataset((XYDataset) Res_dataSet, 1), // data
						PlotOrientation.VERTICAL, // orientation
						true, // include legend
						true, // tooltips?
						false // URLs?
						);

				break;

		}
		
		ret_arr[0] = chart;
		ret_arr[1] = Res_dataSet;
		return ret_arr;
	}
	

	/**
	 * Updates the Chart from the current runs in the Projects class
	 * @param myFrame
	 * 					the JFrame of the MainWindow
	 * @param Res_dataSet
	 * 				the dataset of the graph from the mainwindo
	 * @param Res_chart
	 * 				the Chart that sits in the MainWindow
	 * @param Res_chartPanel
	 * 				The ChartPanel that contains the Chart
	 * @param jta5
	 * 				the TextArea in the MainWindow to put plain output into
	 * @param Res_graph
	 * 				the JGraph if it is a graph, instead of a chart
	 */
	public static void updateChart(JFrame myFrame, Dataset Res_dataSet, JFreeChart Res_chart,
			ChartPanel Res_chartPanel, JTextArea jta5, JGraph Res_graph) {

		switch (pwhn.cP.Res_Method) {

			case 0: //data plain text
				jta5.setText("");
				for (int i = 0; i < pwhn.cP.Res_Content.atm.getRowCount(); i++) {
					RunCollection.RunInfo ri = pwhn.cP.Res_Content.getRun(i);
					if (ri.bVisible && ri.Data != null) {
						//some infos about the run
						jta5.append(ri.getInfos());
						//the actual content
						String[] d = ri.Data.split("\n");
						for (int j = 0; j < d.length; j++) {
							if (!d[j].matches("^(\\p{Print}|\\p{Space})*$")) {
								jta5.append("<binary data>\n");
								continue;
							}
						}
						for (int j = 0; j < d.length; j++)
							jta5.append(d[j] + "\n");
						jta5.append("\n");
					}
				}
				break;

			case 1: //XY things
			case 2:
			case 3:
			case 4:
				((XYSeriesCollection) Res_dataSet).removeAllSeries();
				XYItemRenderer xyir = ((XYPlot) Res_chart.getPlot())
						.getRenderer();

				// loop over all series
				for (int i = 0; i < pwhn.cP.Res_Content.atm.getRowCount(); i++) {
					RunCollection.RunInfo ri = pwhn.cP.Res_Content.getRun(i);
					if (ri.bVisible && ri.Data != null) {
						XYSeries ds = new XYSeries(ri.Name);

						//set the paint for this series
						//xyir.setSeriesPaint(i, ri.runColor);

						// and split on \n
						String[] d = ri.Data.split("\n");
						for (int j = 0; j < d.length; j++) {

							String item = d[j];
							item = item.substring(1, item.length() - 1);
							String[] items = item.split(",");
							try {
								double x = new Double(items[0]).doubleValue();
								double y = new Double(items[1]).doubleValue();
								ds.add(x, y);
							} catch (Exception e) {
								// e.printStackTrace();
								JOptionPane.showMessageDialog(
										myFrame,
										"Error converting fields to numbers.");
							}
						}
						((XYSeriesCollection) Res_dataSet).addSeries(ds);
						xyir.setSeriesPaint(((XYSeriesCollection) Res_dataSet)
								.getSeriesCount() - 1, ri.runColor);

					}
				}

				break;
			case 5: //XYZ
			case 6:
				((XYZSeriesCollection) Res_dataSet).removeAllSeries();
				xyir = ((XYPlot) Res_chart.getPlot()).getRenderer();

				// loop over all series
				for (int i = 0; i < pwhn.cP.Res_Content.atm.getRowCount(); i++) {
					RunCollection.RunInfo ri = pwhn.cP.Res_Content.getRun(i);
					if (ri.bVisible && ri.Data != null) {

						XYZSeries xs = new XYZSeries(ri.Name);
						//set paint
						//xyir.setSeriesPaint(i, ri.runColor);

						// and loop over strings in array
						String[] d = ri.Data.split("\n");
						for (int j = 0; j < d.length; j++) {

							String item = d[j];
							item = item.substring(1, item.length() - 1);
							String[] items = item.split(",");

							try {
								int x = (int) Math.round(new Double(items[0])
										.doubleValue());
								int y = (int) Math.round(new Double(items[1])
										.doubleValue());
								double z = 1;
								if (items.length > 2)
									z = new Double(items[2]).doubleValue();
								xs.add(x, y, z);
							} catch (Exception ex) {
								JOptionPane.showMessageDialog(
										myFrame,
										"Error converting fields to numbers.");
							}
						}
						((XYZSeriesCollection) Res_dataSet).addSeries(xs);
						xyir.setSeriesPaint(((XYZSeriesCollection) Res_dataSet)
								.getSeriesCount() - 1, ri.runColor);

					}
				}
				break;

			case 7: //Name,Y
			case 8:
			case 9:
				DefaultCategoryDataset cs = ((DefaultCategoryDataset) Res_dataSet);
				cs.clear();
				CategoryItemRenderer ciir = ((CategoryPlot) Res_chart.getPlot())
						.getRenderer();

				for (int i = 0; i < pwhn.cP.Res_Content.atm.getRowCount(); i++) {

					RunCollection.RunInfo ri = pwhn.cP.Res_Content.getRun(i);
					if (ri.bVisible) {
						String ser = ri.Name;

						// and loop over strings in array
						String[] d = ri.Data.split("\n");
						for (int j = 0; j < d.length; j++) {

							String item = d[j];
							item = item.substring(1, item.length() - 1);
							String[] items = item.split(",");
							double y = new Double(items[1]).doubleValue();

							cs.addValue(y, ser, items[0]);

						}
						ciir
								.setSeriesPaint(
										(cs.getRowCount() - 1),
										ri.runColor);

					}
				}
				break;

			case 10: //path infos plain
				jta5.setText("");
				for (int i = 0; i < pwhn.cP.Res_Content.atm.getRowCount(); i++) {
					RunCollection.RunInfo ri = pwhn.cP.Res_Content.getRun(i);
					if (ri.bVisible && ri.pi != null) {
						jta5.append(ri.getInfos());
						jta5.append(ri.pi.toString());
					}
				}
				break;

			case 11: //path info link RTT vs. size histo
				((XYSeriesCollection) Res_dataSet).removeAllSeries();
				xyir = ((XYPlot) Res_chart.getPlot()).getRenderer();

				// loop over all series
				for (int i = 0; i < pwhn.cP.Res_Content.atm.getRowCount(); i++) {
					RunCollection.RunInfo ri = pwhn.cP.Res_Content.getRun(i);
					if (ri.bVisible && ri.pi != null) {
						XYSeries ds = new XYSeries(ri.Name);

						//traverse the path
						Visitors.DelayHistoVisitor dhv = new Visitors().new DelayHistoVisitor();
						JGHelper.traversePath(ri.pi, dhv, 0);
						double max = dhv.max;

						double[] values = new double[(int) max + 1];
						//System.out.println("max:"+max);
						//and add all the els from the HT to an array
						for (Enumeration e = dhv.ht.keys(); e.hasMoreElements();) {
							try {
								int x = ((Double) e.nextElement()).intValue();
								double y = ((Double) dhv.ht.get(new Double(x)))
										.doubleValue();
								values[x] = y;
								//System.out.println("adding ("+x+","+y+")");

							} catch (Exception ex) {
								ex.printStackTrace();
								JOptionPane.showMessageDialog(
										myFrame,
										"Error while traversing path.");
								return;
							}
						}
						//now we need to only run backwards over the array and add all the values up
						//to form a histogram
						double running = 0;
						for (int j = (int) max; j >= 0; j--) {
							//add current value to running val
							running += values[j];
							//and add to dataset
							ds.add(j, running);
							//System.out.println("adding 2 line("+j+","+running+")");

						}
						((XYSeriesCollection) Res_dataSet).addSeries(ds);
						xyir.setSeriesPaint(((XYSeriesCollection) Res_dataSet)
								.getSeriesCount() - 1, ri.runColor);

					}
				}

				break;

			case 12://		path info on a 2d world map as a tree in a jgraph instance
			case 13://		FTT tree
			case 14: //		left tree

				//clear graph model
				DefaultGraphModel gm = new DefaultGraphModel();
				
				Res_graph.setModel(gm);

				//if there is more than one tree, this says how much we need to shift down
				int y_inc = 0;

				//				loop over all series
				for (int i = 0; i < pwhn.cP.Res_Content.atm.getRowCount(); i++) {
					RunCollection.RunInfo ri = pwhn.cP.Res_Content.getRun(i);
					if (ri.bVisible && ri.pi != null) {

						//set color of Run, and number of exp. nodes
						JGHelper.setThings(
								ri.runColor,
								ri.pi.numNodes,
								pwhn.cP.Res_Method,
								y_inc);

						//create the cells recursively
						if (pwhn.cP.Res_Method == 12)
							JGHelper.rec_createCells(
									ri.pi,
									null,
									pwhn.cP.Res_Method,
									0,
									0);
						else {
							JGHelper.DFS_createGraph(
									ri.pi,
									null,
									pwhn.cP.Res_Method,
									0);
							//set X_START for next tree accordingly
							y_inc = (JGHelper.max_level + 1) * JGHelper.Y_STEP;
						}

						//and add to model
						gm.insert(
								JGHelper.ar.toArray(),
								JGHelper.atts,
								JGHelper.cs,
								null,
								null);

					}
				}

				break;
		}

		ChartPrefs.show_prefs(false);
		myFrame.repaint();
		Res_chartPanel.repaint();
	}
	
}
