package utep.plmr;

import java.io.*;

/**
 * Class that executes a given commandline
 * accepts a stdin as byte[] and returns stdout as byte[]
 * can notify an object upon termination
 * implements interfaces for Thread and Rice.Executable
 * @author derDoc
 *
 */public class CmdExecer extends Thread implements rice.Executable {

	/**cmd to exec */
	public String	cmd		= "";

	/**its stdin */
	public byte[]	stdin	= new byte[0];

	/**the captured stdout */
	public byte[]	stdout	= new byte[0];

	/**set if someone wants to be notified after termination */
	public Object	notifyThis;

	/**runtime in ms */
	public long		runTime;

	/**true = output msgs */
	public boolean	bEcho	= true;

	/**
	 * creates a new commandExecer
	 * @param c
	 * 			the commandline
	 * @param in
	 * 			the stdin
	 * @param o
	 * 			Object to notify
	 */
	public CmdExecer(String c, byte[] in, Object o) {
		cmd = c;
		stdin = in;
		notifyThis = o;
	}

	/**
	 * Runs this instance and returns the stdout as an obj, impl. from Rice.Executable
	 * @return stdout as byte[]
	 */
	public Object execute() {
		run();
		return stdout;
	}

	/**
	 * Runs this instance, implementation from Thread
	 */
	public void run() {
		try {
			Thread.sleep(0);
			if (bEcho)
				System.out.println(this.getName() + ": execing: '" + cmd + "'");
			if (stdin != null && bEcho)
				System.out.println(this.getName() + ": with stdin: '"
						+ new String(stdin) + "'");

			long s = System.currentTimeMillis();
			Process p = Runtime.getRuntime().exec(cmd);
			BufferedOutputStream out = new BufferedOutputStream(p
					.getOutputStream());

			if (stdin != null)
				out.write(stdin);
			out.close();

			int exitval = p.waitFor();

			runTime = System.currentTimeMillis() - s;

			BufferedInputStream in = new BufferedInputStream(p.getInputStream());
			byte[] line = new byte[in.available()];

			in.read(line);

			in.close();

			if (exitval == 0) {
				if (bEcho)
					System.out.println(this.getName() + " code(" + exitval
							+ "): " + new String(line));
				if (line.length > 0)
					this.stdout = line;
			} else {
				BufferedInputStream err = new BufferedInputStream(p
						.getErrorStream());
				byte[] eline = new byte[err.available()];

				err.read(eline);

				err.close();
				if (bEcho) {
					System.out.println(this.getName() + " code(" + exitval
							+ "):" + new String(line));
					System.out.println(this.getName() + "err:"
							+ new String(eline));
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		if (this.notifyThis != null) {
			synchronized (this.notifyThis) {
				this.notifyThis.notify();
			}
		}
	}
}
