package utep.plmr;

import java.awt.Color;
import java.awt.Font;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Map;

import javax.swing.BorderFactory;

import org.jgraph.graph.AttributeMap;
import org.jgraph.graph.ConnectionSet;
import org.jgraph.graph.DefaultEdge;
import org.jgraph.graph.DefaultGraphCell;
import org.jgraph.graph.GraphConstants;

import utep.plmr.server.PathInfo;

/**
 * class with static helper functions for creation of graphs
 * @author derDoc
 *
 */
public class JGHelper {

	//some defs
	public static final int		FONT_SIZE		= MainWindow.GRAPH_FONT_S;

	public static final int		CELL_WID			= 20;

	public static final int		CELL_HEI			= 7;

	public static final int		X_STEP			= CELL_WID + 50;

	public static final int		Y_STEP			= CELL_HEI + 50;

	public static int			X_START			= 5;

	public static int			Y_START			= 5;

	public static int			TREE_SCALE		= 1;

	//defs for types of graph to create
	public static final int		WORLD_MAP_2D		= 12;

	public static final int		FTT_TREE			= 13;

	public static final int		LEFT_TREE		= 14;

	//static thingies for all funcs
	/** Color of nodes to create */
	public static Color			c;
	/** List of all nodes for Graph */
	public static ArrayList		ar;
	/** St of Connections */
	public static ConnectionSet	cs;
	/** Map of attributes */
	public static Map			atts;
	/** Style for Arrows */
	public static AttributeMap	arrowStyle;
	/** X-deltas for each level in an array for next node to create in that level */
	public static int			level_xs[];
	/** max level seen  */
	public static int			max_level;

	/**
	 * Sets the necc. things for creating cells in the graph once and for all
	 * @param color
	 * 			color of cells and edges
	 * @param type
	 * 			Type of tree to create, see consts.
	 * @param max
	 * 			number of cells to expect
	 * @param y_inc
	 * 			number of pixels to inc the y coord
	 * 			used when drawing more than one graph in a window 
	 */
	public static void setThings(Color color, int max, int type, int y_inc) {

		//		TL Containers for the graphModel
		cs = new ConnectionSet();
		atts = new Hashtable();
		//array of objs
		ar = new ArrayList(max);

		//set the color into a line style
		arrowStyle = new AttributeMap();
		GraphConstants.setLineBegin(arrowStyle, GraphConstants.ARROW_TECHNICAL);
		GraphConstants.setBeginFill(arrowStyle, true);
		//GraphConstants.setBeginSize(arrowStyle, MainWindow.GRAPH_FONT_S);
		//GraphConstants.setEndSize(arrowStyle, MainWindow.GRAPH_FONT_S);
		GraphConstants.setLineWidth(arrowStyle, 1);
		GraphConstants.setLineColor(arrowStyle, color);
		GraphConstants.setForeground(arrowStyle, color);
		GraphConstants.setFont(arrowStyle, GraphConstants.DEFAULTFONT
				.deriveFont(Font.PLAIN, MainWindow.GRAPH_FONT_S * 5));

		JGHelper.level_xs = null;
		JGHelper.c = color;
		max_level = 0;
		Y_START += y_inc;
		if (y_inc == 0)
			Y_START = 5;
		//set scale for cells if tree type
		if (type > 12)
			TREE_SCALE = 3;
		else
			TREE_SCALE = 1;
	}

	/**
	 * recursively DFS-traverses the PathInfo and creates a tree in the current
	 * objs for later insertion into an jgraph instance
	 * @param p
	 * 			the PathInfo
	 * @param parent
	 * 			parent to connect current cell to, if any
	 * @param type
	 * 			type of tree to create, see constants
	 * @param level
	 * 			level we are currently in in the tree
	 */
	public static void DFS_createGraph(PathInfo p, DefaultGraphCell parent,
			int type, int level) {

		if (level_xs == null) {
			level_xs = new int[p.numNodes];

			for (int i = 0; i < level_xs.length; i++)
				level_xs[i] = X_START;
		}

		
		if (level > max_level)
			max_level = level;
		//switch on type of tree (FTT || LEFT)
		switch (type) {
			case FTT_TREE:
				//in this case we just draw ourselves at the provided level at
				// (y, level_xs[lvl]) ,
				//and then recurse over our children, resulting in a DFS

				//draw self
				DefaultGraphCell dgc = rec_createCells(
						p,
						parent,
						type,
						JGHelper.level_xs[level],
						Y_START + (level * Y_STEP));

				//advance x-counter for level
				JGHelper.level_xs[level] += X_STEP;

				
				//and recurse over children
				for (int j = 0; j < p.children.length; j++)
					DFS_createGraph(p.children[j], dgc, type, level + 1);

				break;

			case LEFT_TREE:
				//in this case we need to draw ourselves all the way down to the leaf,
				//i.e. call recursively until level is numChildren+1,
				//and then recurse back

				//draw self first
				dgc = rec_createCells(p, parent, type, level_xs[level], Y_START
						+ (level * Y_STEP));
				//				advance x-counter for level
				level_xs[level] += X_STEP;

				//if not leaf yet, i.e. level < numChils: recurse down
				if (level < p.children.length) {
					DFS_createGraph(p, dgc, type, level + 1);
					//and recurse into children if not leaf, ie start at numChilds-1
					//since children are not ordered, we simply assume first one in the list is oldest
					//ie. closest to us, thus list has to be traversed reverse 
					DFS_createGraph(
							p.children[p.children.length - 1 - level],
							dgc,
							type,
							level + 1);

				}

				break;

		}

	}

	/**
	 * Recursively traverses the path and adds the cells, if type == 12
	 * connecting them to the parent if given
	 * if type != 12 just creates the given cell from p
	 * options such as color are set once using setThings
	 * 
	 * @param p
	 * 			the PathInfo to get the info about current cell to add
	 * @param parent
	 * 			parent cell
	 * @param type
	 * 			visual Type of tree to create, see constants
	 * @param x
	 * 			x coord to create cell at
	 * @param y
	 * 			y coord to create cell at
	 * @return DefaultGraphCell
	 * 			The created cell for later use
	 */
	public static DefaultGraphCell rec_createCells(PathInfo p,
			DefaultGraphCell parent, int type, int x, int y) {

		// the cell itself
		DefaultGraphCell gc = new DefaultGraphCell(p.node_IP.getHostAddress());
		ar.add(gc);

		//now set all the attribs for gc
		AttributeMap map = new AttributeMap();
		GraphConstants.setBorder(map, BorderFactory.createRaisedBevelBorder());
		GraphConstants.setBackground(map, c.darker().darker());
		GraphConstants
				.setGradientColor(map, c.brighter().brighter().brighter());
		GraphConstants.setForeground(map, Color.white);
		GraphConstants.setFont(map, GraphConstants.DEFAULTFONT.deriveFont(
				Font.PLAIN,
				FONT_SIZE * TREE_SCALE));
		GraphConstants.setOpaque(map, true);

		//set the position, from pathinfo if WORLD_MAP
		if (type == WORLD_MAP_2D) {
			if (p.lon != null)
				x = Math.round((Float.parseFloat(p.lon) + 180)
					* MainWindow.GRPH_BACKGR_SCALE);
				
			if (p.lat != null)
				y = Math.round(((Float.parseFloat(p.lat) * (-1)) + 90)
					* MainWindow.GRPH_BACKGR_SCALE);
		}

		GraphConstants.setBounds(map, map.createRect(x, y, CELL_WID
				* TREE_SCALE, CELL_HEI * TREE_SCALE));
		//and put
		atts.put(gc, map);

		//add an incoming port for children
		gc.addPort(null, "incoming/Center");

		//now make an edge if there is a parent
		if (parent != null) {
			//add another port for outgoing edge
			gc.addPort(null, "outgoing/Center");

			DefaultEdge edge = new DefaultEdge(new Long(p.delay2parent));
			cs.connect(edge, parent.getChildAt(0), gc.getChildAt(1));
			ar.add(edge);
			atts.put(edge, arrowStyle);
		}

		//and traverse children, if worldmap
		if (type == WORLD_MAP_2D) {
			for (int j = 0; j < p.children.length; j++)
				rec_createCells(p.children[j], gc, type, 0, 0);
		}

		return gc;
	}

	/**
	 * BFS traverses a PathInfo instance and calls the visitor given in pv
	 * @param p
	 * 			the PathInfo to traverse
	 * @param pv
	 * 			the visitor interface to call
	 * @param level
	 * 			level in the tree for Visitor
	 */public static void traversePath(PathInfo p, PIVisitor pv, int level) {

		//call with self
		pv.visitPI(p, level);
		
		//and recurse children
		for (int j = 0; j < p.children.length; j++)
			traversePath(p.children[j], pv, level+1);
		
	}
}
