package utep.plmr;

import java.awt.*;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.Serializable;
import java.util.*;
import javax.swing.JOptionPane;
import javax.swing.table.*;
import org.jfree.chart.plot.*;

import utep.plmr.Visitors.StatsBuilderVisitor;
import utep.plmr.server.*;

/**
 * Class that stores info about all Runs Has an arrayList with all Runs
 * as local class RunInfo also exports an AbstractTableModel
 * which in turn gets its info from local RunInfos for the Table of Runs
 * @author  derDoc
 */
public class RunCollection implements Serializable {

	private static final long				serialVersionUID		= 310719802L;

	transient public AbstractTableModel		atm					= new myTableModel();

	private ArrayList						Runs					= new ArrayList();

	private String[]							columnNames			= { "show", 
			"color",	"Name",
			"type", "returned/#jobs", "runtime/ms"					};

	private String[]							TypeNames			= { "Script",
			"Native", "FP-FTT", "FP-KBT", "CO-KBT", "n/a"	};
	
	private DrawingSupplier drawSuppl;

	/**
	 * creates new RunCollection
	 *
	 */
	public RunCollection() {
		this.drawSuppl = new DefaultDrawingSupplier();
	}

	/**
	 * myTableModel
	 * 				extends an AbstractTableModel for the Table in the ResPanel
	 * 				just gets its info from the Array of Runs
	 * @author derDoc
	 *
	 */
	public class myTableModel extends AbstractTableModel {

		public myTableModel() {}

		/** gets the current row count
		 * @return row count
		 */
		public int getRowCount() {
			return Runs.size();
		}

		/**
		 * Column count
		 * @return number of columns
		 */
		public int getColumnCount() {
			return columnNames.length;
		}

		/**
		 * Returns column name to display for a specific column
		 * @param col
		 * 		the index of the col
		 * @return
		 * 		The column's name
		 */
		public String getColumnName(int col) {
			return columnNames[col];
		}

		/**
		 * Gets a value of a specific Cell
		 * @param row
		 * 			row
		 * @param col
		 * 			col
		 * @return
		 * 			The val of the cell as an obj
		 */
		public Object getValueAt(int row, int col) {
			RunInfo ri = (RunInfo) Runs.get(row);
			switch (col) {
				case 0:
					return new Boolean(ri.bVisible);
				case 1:
					return ri.runColor;
				case 2:
					return ri.Name;
				case 3:
					return TypeNames[ri.type];
				case 4:
					if (ri.totalJobs < 0)
						return "n/a";
					return new String(ri.completedJobs + "/" + ri.totalJobs);
				case 5:
					if (ri.millis < 0)
						return "n/a";
					return new Long(ri.millis);
			}
			return null;
		}

		/**
		 * Sets a new val for a cell
		 */
		public void setValueAt(Object value, int row, int col) {
			RunInfo ri = (RunInfo) Runs.get(row);

			switch (col) {
				case 0:
					ri.bVisible = ((Boolean) value).booleanValue();
					break;
				case 1:
					ri.Name = (String) value;
					break;
				case 2:
					ri.millis = ((Long) value).longValue();
					break;
			}
			this.fireTableCellUpdated(row, col);
			MainWindow.updateChart();
			MainWindow.myFrame.repaint();
		}

		/**
		 * Gets a Column class to decide which renderer to use
		 */
		public Class getColumnClass(int c) {
			return getValueAt(0, c).getClass();
		}

		/**
		 * Specifies if a cell is editable
		 */
		public boolean isCellEditable(int row, int col) {
			return (col == 0);
		}

	}

	/**
	 * Adds a Run to the list of runs stored in this instance
	 * @param n
	 * 			the Name of the Run
	 * @param d
	 * 			content as byte[]
	 * @param s
	 * 			millis exec time
	 * @param t
	 * 			Type
	 * @param j
	 * 			Count jobs total
	 * @param c
	 * 			count completed jobs
	 * @return
	 * 			index of newly created run
	 */
	public int addRun(String n, byte[] d, long s, int t, int j, int c) {
		this.Runs.add(new RunInfo(n, d, s, true, t, j, c));
		atm.fireTableRowsInserted(Runs.size() - 1, Runs.size() - 1);
		if (d != null && null != MainWindow.myFrame) {
			MainWindow.updateChart();
			MainWindow.myFrame.repaint();
		}
		return (this.Runs.size() - 1);
	}

	/**
	 * Like Above addRun only with content as String
	 * @param n
	 * @param d
	 * @param s
	 * @param t
	 * @param j
	 * @param c
	 * @return
	 */
	public int addRun(String n, String d, long s, int t, int j, int c) {
		this.Runs.add(new RunInfo(n, d, s, true, t, j, c));
		atm.fireTableRowsInserted(Runs.size() - 1, Runs.size() - 1);
		if (d != null && null != MainWindow.myFrame) {
			MainWindow.updateChart();
			MainWindow.myFrame.repaint();
		}
		return (this.Runs.size() - 1);
	}

	/**
	 * Allows creation of Run while specifying the PathInfo and client bytes in directly
	 * @param n
	 * @param d
	 * @param s
	 * @param t
	 * @param j
	 * @param c
	 * @param p
	 * 		PathInfo of Run to add
	 * @param len
	 * 		Sum cleint bytes in
	 * @return
	 */
	public int addRun(String n, byte[] d, long s, int t, int j, int c, PathInfo p, long len) {
		int i = this.addRun(n,d,s,t,j,c);
		this.getRun(i).pi = p;
		this.getRun(i).sumClientIn = len;
		return i;
	}
	
	/**
	 * Adds a PathInfo to a run
	 * @param i
	 * 		The inex of run
	 * @param pi
	 * 		PI to add
	 */
	public void addPathInfo(int i, PathInfo pi){
		this.getRun(i).pi = pi;
	}
	
	/**
	 * Updates a run
	 * @param i
	 * 			Index
	 * @param addTo
	 * 			bytes to add to Content
	 * @param s
	 * 			New millis
	 * @param c
	 * 			completed jobs count delta
	 * @param addToIn
	 * 			Sum of client bytes in
	 * @param addToExecT
	 * 			Sum of all exec times of child CmdExecer's
	 * @throws IOException
	 */
	public void updateRun(int i, byte[] addTo, long s, int c, long addToIn, long addToExecT)
			throws IOException {
		RunInfo ri = (RunInfo) Runs.get(i);
		ri.completedJobs += c;
		if (s > 0)
			ri.millis = s;
		if (addTo != null && addTo.length > 0) {
			ri.Data += new String(addTo)
					+ ((addTo[addTo.length - 1] == '\n') ? "" : "\n");
			ByteArrayOutputStream b = new ByteArrayOutputStream();
			b.write(ri.bData);
			b.write(addTo);
			ri.bData = b.toByteArray();

		}
		
		ri.sumClientIn += addToIn;
		ri.sumLinkExecTimes += addToExecT;
		
		atm.fireTableRowsUpdated(i, i);
		if (null != MainWindow.myFrame) {
			MainWindow.updateChart();
			MainWindow.myFrame.repaint();
		}
	}

	/** Deletes a run from the table
	 * 
	 * @param row
	 * 			Its index
	 */
	public void delRun(int row) {
		this.Runs.remove(row);
		atm.fireTableRowsDeleted(row, row);
		if (null != MainWindow.myFrame) {
			MainWindow.updateChart();
			MainWindow.myFrame.repaint();
		}
	}

	/**
	 * returns a specific run
	 * @param i
	 * 		the run's index in the array
	 * @return
	 * 		the RunInfo
	 */
	public RunInfo getRun(int i) {
		return (RunInfo) Runs.get(i);
	}

	/** Returns all runs as an array of RunInfos
	 * @return  the runs
	 * @uml.property  name="runs"
	 */
	public RunInfo[] getRuns() {
		return ((RunInfo[]) Runs.toArray(new RunInfo[0]));
	}

	/**
	 * Changes the Series with the Category for Category Runs
	 *
	 */
	public void XChangeSerCat() {

		ArrayList newR = new ArrayList();
		RunInfo[] rs = getRuns();
		String newdata;

		// Run's a 3-dim array, so data[0][0] of every run is new name for run
		// outermost loop needs to run over every element of all runs
		try {
			// take size of first run
			String[] d = rs[0].Data.split("\n");

			for (int i = 0; i < d.length; i++) {
				// get name of run from run0[i][0]
				String run0item = d[i];
				run0item = run0item.substring(1, run0item.length() - 1);
				String[] run0items = run0item.split(",");
				String run_name = run0items[0];

				newdata = "";
				// inner loop needs to run over every ith el. of all runs
				for (int j = 0; j < rs.length; j++) {
					// if current run does not have that element, quit
					String[] jd = rs[j].Data.split("\n");
					if (jd.length <= i) {
						JOptionPane.showMessageDialog(
								null,
								"Number of elements in Runs do not match");
						return;
					}
					String item = jd[i];
					item = item.substring(1, item.length() - 1);
					String[] items = item.split(",");
					// if name is not the same, exit
					if (!run_name.equals(items[0])) {
						JOptionPane.showMessageDialog(
								null,
								"Names of elements do not match");
						return;
					}
					// build string for this element from name of run
					String el = ("(" + rs[j].Name + "," + items[1]);
					if (items.length > 2)
						el += "," + items[2];
					el += ")\n";
					newdata += el;
				}
				newR.add(new RunInfo(run_name, newdata, -1, true, 5, -1, -1));
			}
			// and change this.runs
			this.Runs = newR;
			atm.fireTableDataChanged();
		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	/**
	 * Class that holds all information about a particular run
	 * @author  derDoc
	 */
	public class RunInfo implements Serializable {
		private static final long	serialVersionUID	= 310719803L;

		/**the name of the run */
		public String				Name				= "";
		/**if it is visible */
		public boolean				bVisible;

		/**the data that the run returned as a string */
		public String				Data				= "";
		
		/**the same, only as binary data this time */
		public byte[]				bData				= new byte[0];

		/**time the run took in milliseconds */
		public long					millis;

		/**type of the run */
		public int					type;

		/**total jobs of the run */
		public int					totalJobs;

		/**completed jobs in this run */
		public int					completedJobs;

		/**the pathinfo object from the tree run, if any */
		public PathInfo				pi;
		
		/**the paint used to paint this run */
		public Color					runColor;
		
		/** Sum of Bytes of client received */
		public long					sumClientIn;
		/** Exec times of all child Execers summed */
		public long					sumLinkExecTimes;
		

		/**
		 * Create new RunInfo
		 * @param n
		 * 			Name
		 * @param s
		 * 			exec time in millis
		 * @param v
		 * 			Visibily of Run in Res Panel
		 * @param t
		 * 			Type
		 * @param j
		 * 			Total jobs that run was req'ed on
		 * @param c
		 * 			# Completed jobs
		 */
		public RunInfo(String n, long s, boolean v, int t, int j, int c) {
			this.millis = s;
			this.bVisible = v;
			this.Name = n;
			this.type = t;
			this.totalJobs = j;
			this.completedJobs = c;
			
			//set the next paint from the drawing supplier
			this.runColor = (Color)drawSuppl.getNextPaint();
			
		}

		/**
		 * Creates new Run, same as above except allows specifying contents
		 * @param n
		 * @param d
		 * 			The result of the run as byte array, string data is set autom.
		 * @param s
		 * @param v
		 * @param t
		 * @param j
		 * @param c
		 */
		public RunInfo(String n, byte[] d, long s, boolean v, int t, int j,
				int c) {
			this(n, s, v, t, j, c);

			if (d != null) {
				this.bData = d;
				this.Data = new String(d);
			}
		}

		/**
		 * Creates new Run, same as above except allows specifying contents as String
		 * @param n
		 * @param d
		 * 			The result of the run as a String, bytes are set autom.
		 * @param s
		 * @param v
		 * @param t
		 * @param j
		 * @param c
		 */
		public RunInfo(String n, String d, long s, boolean v, int t, int j,
				int c) {
			this(n, s, v, t, j, c);

			if (d != null) {
				this.bData = d.getBytes();
				this.Data = d;
			}
		}
		
		/**
		 * Extra PathInfos about a run in textual form 
		 * @return
		 * 		the extra info as a String
		 */
		public String getInfos() {

			String temps = "'" + Name + "'" + ":\n" + "Time:\t\t" + millis
					+ "\n" + "Avg. ms/hop:\t\t"
					+ ((float) millis / completedJobs) + "\n"
					+ "Client bytes in:\t\t" + (sumClientIn) + "\n"
					+ "% answered/all asked:\t"
					+ ((float) completedJobs * 100 / totalJobs) + "%\n";

			switch (type) {
				case 1: //native
					temps += "Avg. bytes/link:\t"
							+ ((float) sumClientIn / completedJobs) + "\n"
							+ "Avg. bytes/msec:\t"
							+ ((float) sumClientIn / sumLinkExecTimes) + "\n"
							+ "Avg. link delay:\t"
							+ ((float) sumLinkExecTimes / completedJobs)
							+ "\n" + "Avg. fan-in:\t\t" + (completedJobs) + "\n"
							+ "Avg. path length:\t1\n";
					break;

				case 2:
				case 3:
				case 4:
					if (pi != null) {
						//add ring answered
						temps += "% answered/ring size:\t"
								+ ((float) completedJobs * 100 / pi.ringSize)
								+ "%\n";
						//build stats for pi
						Visitors.StatsBuilderVisitor sbv = new Visitors().new StatsBuilderVisitor();
						JGHelper.traversePath(pi, sbv, 0);
						temps += sbv.toString() + "\n";
					}
					break;

			}
			return temps;
		}
	}

}
