package utep.plmr;

import java.io.*;
import java.net.*;
import java.nio.*;
import java.text.DateFormat;
import java.util.*;
import javax.swing.*;
import rice.Continuation;
import utep.plmr.server.*;
import utep.plmr.server.messages.*;

/**
 * The Class that takes care of actually running a Run
 * has private classes for differnt types of runs
 * and all neccessary variables for state
 * 
 * @author derDoc
 *
 */
public class Runners {

	public final static int		SYNC				= 0;

	public final static int		EXEC				= 1;

	public static final String	SCRIPT_INTERPRETER	= "python";

	public static final String	SCRIPT_NAME			= "PLMR";

	public static final String	SCRIPT_EXT			= ".py";

	public static final String	NATIVE_RSH			= "ssh";

	public static final String	NATIVE_SYNC			= "rsync";

	public static final int		MAX_DESER_TRIES		= 75;

	public static final int		RES_MSG_SIZE		= 1024;

	public static final String	HASHER_SCRIPT		= "python hasher.py";

	public static final String	S3in1_cmdlines[]	= { "gen", "agg", "eval" };

	//changing vars

	public int					method;

	public volatile boolean		bRunning;

	public JProgressBar			theProgress;

	public String				RunName;

	public Thread				theR;

	public Runners() {}

	/**
	 * Constructor for a Runner
	 * 
	 * @param type
	 * 			The type of tree requested, int from JComboBox from MainWindow
	 * @param jpb
	 * 			The ProgressBar to update
	 * @param name
	 * 			Name for the run
	 * 			if null, sync is assumed
	 * 			if "Name" the current Date and Time are substituted
	 */
	public Runners(int type, JProgressBar jpb, String name) {
		this.theProgress = jpb;
		method = EXEC;

		if (name == null) {
			method = SYNC;
		}

		else if (!name.equals("Name"))
			this.RunName = name;

		Runnable r = null;

		//check synced state
		if (method == SYNC && pwhn.cP.synced) {
			if (JOptionPane.NO_OPTION == JOptionPane.showConfirmDialog(
					null,
					"You are about to sync, yet the current project seems in "
							+ "sync with this set of IPs, are you sure?",
					"Sync?",
					JOptionPane.YES_NO_OPTION,
					JOptionPane.INFORMATION_MESSAGE)) {
				stop();
				return;
			}
		} else if (type < 2 && method == EXEC && !pwhn.cP.synced) {
			int o = JOptionPane
					.showConfirmDialog(
							null,
							"You are about to execute a flat run, yet the current project seems out of "
									+ "sync with this set of IPs!\nSync now using native sync?",
							"Sync?",
							JOptionPane.YES_NO_CANCEL_OPTION,
							JOptionPane.INFORMATION_MESSAGE);
			if (o == JOptionPane.CANCEL_OPTION) {
				stop();
				return;
			} else if (o == JOptionPane.YES_OPTION) {
				type = 1;
				method = SYNC;
			}
		} else if (type > 1 && method == EXEC && !pwhn.cP.synced
				&& pwhn.cP.Init_FileIsDir) {
			int o = JOptionPane
					.showConfirmDialog(
							null,
							"You are about to execute an unsynced tree run "
									+ "with a directory, however tree runs are only capable of "
									+ "transporting single files!\nSync now?",
							"Sync?",
							JOptionPane.YES_NO_CANCEL_OPTION,
							JOptionPane.INFORMATION_MESSAGE);
			if (o == JOptionPane.CANCEL_OPTION) {
				stop();
				return;
			} else if (o == JOptionPane.YES_OPTION) {
				type = 1;
				method = SYNC;
			}
		} else if (type > 1 && method == SYNC && !pwhn.cP.Init_FileIsDir) {
			int o = JOptionPane
					.showConfirmDialog(
							null,
							"You are about to sync single files using a tree method, "
									+ "since tree methods autosync single files this will execute the run "
									+ "\nAre you sure?",
							"Sure?",
							JOptionPane.YES_NO_OPTION,
							JOptionPane.INFORMATION_MESSAGE);
			if (o == JOptionPane.NO_OPTION) {
				stop();
				return;
			}
		} else if (type > 1 && method == SYNC && pwhn.cP.Init_FileIsDir) {
			int o = JOptionPane.showConfirmDialog(
					null,
					"You are about to sync a directory using a tree method, "
							+ "which can only autosync single files!"
							+ "\nSync now using native sync?",
					"Sync?",
					JOptionPane.YES_NO_CANCEL_OPTION,
					JOptionPane.INFORMATION_MESSAGE);
			if (o == JOptionPane.CANCEL_OPTION) {
				stop();
				return;
			} else if (o == JOptionPane.YES_OPTION) {
				type = 1;
				method = SYNC;
			}
		}

		switch (type) {
			case 0: // script
				r = new ScriptRunner();
				break;
			case 1: // native
				r = new FlatNativeRunner();
				break;
			case 2: //tree FP-FTT
				r = new TreeRunner();
				break;
			case 3: //tree FP-KBT
				r = new TreeRunner();
				((TreeRunner) r).type = BuildMessage.FP_KBT;
				break;
			case 4: //tree CO-KBT
				r = new TreeRunner();
				((TreeRunner) r).type = BuildMessage.CO_KBT;
				break;
		}
		theR = new Thread(r);
	}

	/**
	 * Stops this Runner
	 * sets its running var to false and interrupts all threads
	 * resets the buttons to their original state
	 *
	 */
	public void stop() {
		if (theR != null) {
			this.bRunning = false;
			theR.interrupt();
		}

		MainWindow.jb1a.setText("Run!");
		MainWindow.jb1b.setText("Sync");
		theProgress.setIndeterminate(false);
		theProgress.setValue(0);
	}

	/**
	 * Starts execution of this Runner
	 *
	 */
	public void start() {
		if (theR != null) {
			this.bRunning = true;
			theR.start();
		}
	}

	/* the runners that do the actual work, one for each method */
	/**
	 * ScriptRunner
	 * 				Implements execution by executing the pwhn.py script
	 * 				prepares all files and switches and kicks pwhn.py off in quit mode
	 * 				waits for it to finished and returns the result
	 */
	private class ScriptRunner implements Runnable {

		/**
		 * Runs this instance
		 */
		public void run() {

			try {
				theProgress.setIndeterminate(true);
				String cmd = "";
				// System.out.println(cP.toString());
				cmd += SCRIPT_INTERPRETER + " ";
				cmd += SCRIPT_NAME + SCRIPT_EXT;
				cmd += " -q";
				if (method == SYNC)
					cmd += " -l";
				else if (method == EXEC)
					cmd += " -s";

				cmd += " -t " + pwhn.cP.IP_Threads;

				if (pwhn.cP.IP_Username != null)
					cmd += " -u " + pwhn.cP.IP_Username;

				if (pwhn.cP.IP_File != null)
					cmd += (" -a " + pwhn.cP.IP_File);
				else {
					JOptionPane.showMessageDialog(
							null,
							"Error, no IP file specified");
					return;
				}

				if (pwhn.cP.Init_FileIsDir) {
					cmd += (" -D " + pwhn.cP.Init_File);
					if (pwhn.cP.Init_FileInDir != null)
						cmd += (" -i " + pwhn.cP.Init_FileInDir);
					if (pwhn.cP.Upd_FileInDir != null)
						cmd += (" -g " + pwhn.cP.Upd_FileInDir);
					if (pwhn.cP.Eval_FileInDir != null)
						cmd += (" -e " + pwhn.cP.Eval_FileInDir);
				} else {
					cmd += (" -i " + pwhn.cP.Init_File);
				}

				System.out.println("Execing: " + cmd);

				Thread.sleep(0);
				long start = System.currentTimeMillis();

				Process p = Runtime.getRuntime().exec(cmd);
				int exitval = p.waitFor();
				long stop = System.currentTimeMillis() - start;

				if (exitval == 0) {
					BufferedInputStream in = new BufferedInputStream(p
							.getInputStream());
					byte[] line = new byte[in.available()];
					in.read(line);
					in.close();

					//if no name is given, make one out of the date and repl. all , with ;
					if (RunName == null)
						RunName = DateFormat.getDateTimeInstance().format(
								new Date()).replace(',', ';');

					if (method == EXEC)
						pwhn.cP.Res_Content
								.addRun(RunName, line, stop, 0, 1, 1);
					else
						pwhn.cP.synced = true;

				} else {
					JOptionPane.showMessageDialog(
							null,
							"Something went wrong, script exited with code "
									+ exitval);
				}

			} catch (Exception e) {
				if (bRunning)
					e.printStackTrace();
				else
					System.out.println("Stopping...");
			}
			pwhn.stop();
		}
	} // end scriptrunner

	/**
	 * FlatNativeRunner Implements execution in native Java Runs all the ssh, rsync and execution of scripts in 100% native Java code in subshells using Runtime.exec() thus it is able to give progress on the ProgressBar and also adds content to the runs while they are running
	 * @author  derDoc
	 */
	public class FlatNativeRunner implements Runnable {

		String			user			= "";

		boolean			bShortcut		= false;

		int				nt				= pwhn.cP.IP_Threads;

		Hashtable		mappings		= new Hashtable();

		PathInfo		pi				= new PathInfo();

		public String[]	IPs;

		//public String	cmd_line	 = "";

		public String	e_cmdl			= "";

		public String	f_cmdl			= "";

		public String	g_cmdl			= "";

		public String	prefix_cmdline	= "";

		/**
		 * Get username and necc. vars from MainWindow fields before starting
		 * Also get list of server addresses
		 *
		 */
		public void setup() {
			//			set name of run, if not already set
			if (RunName == null)
				RunName = DateFormat.getDateTimeInstance().format(new Date())
						.replace(',', ';');

			//set username
			if (pwhn.cP.IP_Username != null
					&& !("".equals(pwhn.cP.IP_Username)))
				user = pwhn.cP.IP_Username + "@";

			// get the list of IPs
			IPs = pwhn.cP.IP_ScriptContent.split("\n");
			List ips = new ArrayList(IPs.length);

			for (int i = 0; i < IPs.length; i++)
				if (!IPs[i].startsWith("#"))
					ips.add(IPs[i]);
			IPs = (String[]) ips.toArray(new String[0]);

			//and set running
			bRunning = true;

		}

		/**
		 * starts execution of this instance
		 */
		public void run() {

			theProgress.setValue(0);
			String e = null, f = null, g = null;
			List outs;
			byte[] finalin, finalout;
			int runIdx = -1;
			setup();

			long start = System.currentTimeMillis();
			//int delta = (Math.round((float) 100 / (float) IPs.length));

			try {

				// just sync
				if (method == SYNC) {
					String file;
					if (pwhn.cP.Init_FileIsDir) {
						file = pwhn.cP.Init_File;
						if (file.endsWith("/"))
							file = file.substring(0, file.length() - 1);
					} else
						file = pwhn.cP.Init_File;
					//set length of job
					if (theProgress != null)
						theProgress.setMaximum(IPs.length);
					sync(file);

					// execute
				} else {
					// init, determine file name to exec
					if ("".equals(pwhn.cP.Init_File))
						return;
					e = new File(pwhn.cP.Init_File).getName();
					if (pwhn.cP.Init_FileIsDir) {
						e = e + "/" + pwhn.cP.Init_FileInDir;
					}

					if (pwhn.cP.Init_File3in1) {
						f = pwhn.cP.Init_File;
						if (pwhn.cP.Init_FileIsDir) {
							f = f + "/" + pwhn.cP.Init_FileInDir;
						}
						g = f;
						e_cmdl = S3in1_cmdlines[0];
						f_cmdl = S3in1_cmdlines[1];
						g_cmdl = S3in1_cmdlines[2];

					} else {
						// filename for aggregate
						if (pwhn.cP.Init_FileIsDir) {
							if ((pwhn.cP.Upd_FileInDir != null && !""
									.equals(pwhn.cP.Upd_FileInDir)))
								f = pwhn.cP.Upd_File + "/"
										+ pwhn.cP.Upd_FileInDir;
						} else if (!"".equals(pwhn.cP.Upd_File)) {
							f = pwhn.cP.Upd_File;
						}

						// filename for eval
						if (pwhn.cP.Init_FileIsDir) {
							if ((pwhn.cP.Eval_FileInDir != null && !""
									.equals(pwhn.cP.Eval_FileInDir)))
								g = pwhn.cP.Eval_File + "/"
										+ pwhn.cP.Eval_FileInDir;
						} else if (!"".equals(pwhn.cP.Eval_File)) {
							g = pwhn.cP.Eval_File;
						}
					}
					// get Idx for shortcut updating
					runIdx = pwhn.cP.Res_Content.addRun(
							RunName,
							(String) null,
							0l,
							1,
							IPs.length,
							0);

					// if !agg&&!eval, do shortcutting
					if ((f == null && g == null)) {
						bShortcut = true;
						if (theProgress != null)
							theProgress.setMaximum(IPs.length);
						exec(e, runIdx, start);
					} else {
						if (theProgress != null)
							theProgress.setMaximum(IPs.length * 2);
						outs = exec(e, runIdx, start);
						if (f != null)
							finalin = upd(outs, f);
						else {
							ByteArrayOutputStream baos = new ByteArrayOutputStream();

							for (int i = 0; i < outs.size(); i++) {
								byte[] t = (byte[]) outs.get(i);
								baos.write(String.valueOf(t.length).getBytes());
								baos.write('\n');
								baos.write(t);
								baos.write('\n');
							}
							finalin = baos.toByteArray();
						}
						if (g != null)
							finalout = eval(finalin, g);
						else
							finalout = finalin;

						//and do last update for content
						pwhn.cP.Res_Content.updateRun(
								runIdx,
								finalout,
								0l,
								0,
								0,
								0);
					}
				}

			} catch (Exception ex) {
				if (bRunning)
					ex.printStackTrace();
				else
					System.out.println("Stopping...");
			}

			long stop = System.currentTimeMillis() - start;
			//update the time only
			try {
				if (runIdx > -1)
					pwhn.cP.Res_Content.updateRun(runIdx, null, stop, 0, 0, 0);
			} catch (Exception ex) {
				ex.printStackTrace();
			}
			pwhn.stop();
		}

		/**
		 * Executes the given filename on a set of remote servers using
		 * command set in NATIVE_RSH
		 * @param file
		 * 			Filename to execute
		 * @param rIdx
		 * 			runIdx for shortcut updating
		 * @param start
		 * 			start time in millis
		 * @throws Exception
		 * 			if anything goes wrong which we all dont hope
		 */
		public List exec(String file, int rIdx, long start) throws Exception {
			List cmds = new ArrayList(IPs.length);
			// build command strings
			if (e_cmdl.length() > 0)
				e_cmdl = " " + e_cmdl;

			for (int i = 0; i < IPs.length; i++) {
				String p = NATIVE_RSH + " " + user + IPs[i] + " "
						+ prefix_cmdline + SCRIPT_NAME + "/" + file + e_cmdl;
				cmds.add(p);
			}
			// and loop over cmds
			return loop(cmds, rIdx, start);
		}

		/**
		 * Just rsyncs the given file (dir or file)
		 * @param file
		 * 			the file to sync with rsync
		 * @throws Exception
		 * 			something went wrong
		 */
		public void sync(String file) throws Exception {
			List cmds = new ArrayList(IPs.length);
			// build command strings
			for (int i = 0; i < IPs.length; i++) {
				String p = NATIVE_SYNC + " -arz " + file + " " + user + IPs[i]
						+ ":~/" + SCRIPT_NAME;
				cmds.add(p);
			}

			// and loop over cmds
			loop(cmds, -1, 0l);
			pwhn.cP.synced = true;
		}

		/**
		 * Runs the updator as many times as neccessary
		 * @param l
		 * 			List of byte[] representing the inputs
		 * @param f
		 * 			String - absolute Filename
		 * @return
		 * 			The last output as byte[]
		 * @throws Exception
		 * 			If not found or something else went wrong
		 */
		private byte[] upd(List l, String f) throws Exception {
			byte[] out = (byte[]) l.remove(0);
			
			while (l.size() > 0 && out.length<3) {
				out = (byte[]) l.remove(0);
			}
			
			byte[] ls;
			ByteArrayOutputStream in = new ByteArrayOutputStream();
			if (f_cmdl.length() > 0)
				f_cmdl = " " + f_cmdl;

			//and run over outputs, remove from list until no more
			while (l.size() > 0) {
				in.reset();
				ls = (byte[]) l.remove(0);

				//write length and out to buffer
				in.write(String.valueOf(out.length).getBytes());
				in.write('\n');
				in.write(out);
				in.write('\n');

				//precaution: if len(ls)<3 '()\n': skip
				if (ls.length < 3)
					continue;

				//	write length and out to buffer
				in.write(String.valueOf(ls.length).getBytes());
				in.write('\n');
				in.write(ls);
				in.write('\n');
				//and kick of the cmd
				CmdExecer ce = new CmdExecer(f + f_cmdl, in.toByteArray(), this);
				ce.start();
				//wait for it to finish
				try {
					synchronized (this) {
						this.wait();
					}
				} catch (Exception ex) {
					bRunning = false;
					ce.stop();
					throw (ex);
				}
				//update the progress
				if (theProgress != null)
					theProgress.setValue(theProgress.getValue() + 1);
				//get the output
				out = ce.stdout;
				//and loop
			}
			//return last output
			return out;
		}

		/**
		 * Runs the Evaluator
		 * @param in
		 * 			input for it as byte[]
		 * @param f
		 * 			Filename of it
		 * @return
		 * 			Its output as byte[]
		 * @throws Exception
		 * 			If not found or stopped prematurely
		 */
		public byte[] eval(byte[] in, String f) throws Exception {
			if (g_cmdl.length() > 0)
				g_cmdl = " " + g_cmdl;

			//start eval with input
			CmdExecer ce = new CmdExecer(f + g_cmdl, in, this);
			ce.start();
			try { //wait for it
				synchronized (this) {
					this.wait();
				}
			} catch (Exception ex) {
				bRunning = false;
				ce.stop();
				throw (ex);
			}
			if (theProgress != null)
				theProgress.setValue(100); //finished
			return ce.stdout; //and return
		}

		/**
		 * 	loops over a list of cmds starts as many threads as 'nt' cmd execers
		 *  monitors progress and kicks of new threads if neccess.
		 *  if shortcut is set it calls updateRun when a new result comes in
		 * @param cmds
		 * 			list of commands
		 * @param idx
		 * 			index for updating of runs, if shortcut-able
		 * @param s
		 * 			start time if shortcutting
		 * @return
		 * 			list of byte[] with the outputs
		 * @throws Exception
		 * 			if anything gets wrong 
		 */
		private List loop(List cmds, int idx, long s) throws Exception {
			List tList = new ArrayList();
			List oList = new ArrayList();

			try {
				while (bRunning) {

					Iterator it = tList.iterator();
					// and walk over list removing any deads
					while (it.hasNext()) {
						CmdExecer ce = (CmdExecer) it.next();
						if (!ce.isAlive()) {
							if (idx > -1) {
								long now = System.currentTimeMillis() - s;
								pwhn.cP.Res_Content.updateRun(
										idx,
										((bShortcut) ? ce.stdout : null),
										now,
										1,
										ce.stdout.length,
										ce.runTime);
							}
							if (ce.stdout != null)
								oList.add(ce.stdout);
							it.remove();
							// and advance the progressbar
							//theProgress.setValue(theProgress.getValue() + D);
							if (theProgress != null)
								theProgress
										.setValue(theProgress.getValue() + 1);

						}
					}
					// and check if there is more to do, and some threads left
					if (cmds.size() == 0 && tList.size() == 0)
						break;

					while ((cmds.size() > 0) && (tList.size() < nt)) {
						String p = (String) cmds.remove(0);
						CmdExecer ce = new CmdExecer(p, null, this);
						ce.start();
						tList.add(ce);
					}
					// wait for any to terminate
					synchronized (this) {
						this.wait();
					}
				}
			} catch (Exception ex) {
				if (bRunning)
					ex.printStackTrace();
				else
					System.out.println("Stopping...");

				for (int i = 0; i < tList.size(); i++)
					((Thread) (tList.get(i))).stop();
				bRunning = false;
				throw (ex);
			}
			return oList;
		}

	}//end flatnative runner

	/**
	 * TreeRunner
	 * 			a Runner that implements execution on top of a DHT tree
	 * 			can execute on of three different types of tree
	 * 			packages and sends the buildMessage to the first IP on the list
	 * 			for execution, waits for a ResMgs and returns
	 * @author derDoc
	 *
	 */
	private class TreeRunner implements Runnable {

		public int		type	= BuildMessage.FP_FTT;

		public String	e_cmdl	= "";

		public String	f_cmdl	= "";

		public String	g_cmdl	= "";

		/**
		 * starts this TreeRunner instance
		 */
		public void run() {
			if (theProgress != null)
				theProgress.setIndeterminate(true);

			String[] IPs = pwhn.cP.IP_ScriptContent.split("\n");
			List ips = new ArrayList(IPs.length);
			// build command strings
			for (int i = 0; i < IPs.length; i++)
				if (!IPs[i].startsWith("#"))
					ips.add(IPs[i]);
			IPs = (String[]) ips.toArray(new String[0]);

			long start = System.currentTimeMillis();

			BuildMessage bm = new BuildMessage();
			bm.tree_type = type;
			bm.IPList = IPs;
			bm.bCollPathInfo = pwhn.cP.IP_bCollPI;

			//determine file names
			String e = null, f = null, g = null;

			//init, determine file name to exec
			if ("".equals(pwhn.cP.Init_File))
				return;
			e = new File(pwhn.cP.Init_File).getName();
			if (pwhn.cP.Init_FileIsDir) {
				bm.RSyncBaseDir = SCRIPT_NAME + "/" + e;
				e = e + "/" + pwhn.cP.Init_FileInDir;
			}

			if (!pwhn.cP.Init_File3in1) {
		
				// filename for aggregate
				if (pwhn.cP.Init_FileIsDir) {
					if ((pwhn.cP.Upd_FileInDir != null && !""
							.equals(pwhn.cP.Upd_FileInDir)))
						f = pwhn.cP.Upd_File + "/" + pwhn.cP.Upd_FileInDir;
				} else if (!"".equals(pwhn.cP.Upd_File)) {
					f = new File(pwhn.cP.Upd_File).getName();
				}
				// filename for eval
				if (pwhn.cP.Init_FileIsDir) {
					if ((pwhn.cP.Eval_FileInDir != null && !""
							.equals(pwhn.cP.Eval_FileInDir)))
						g = pwhn.cP.Eval_File + "/" + pwhn.cP.Eval_FileInDir;
					System.out.println("eval file dir=" + g);
				} else if (!"".equals(pwhn.cP.Eval_File)) {
					g = pwhn.cP.Eval_File;
					System.out.println("eval file=" + g);
				}
			}else
				bm.bFile3in1 = true;

			//insert the filenames
			if (e != null)
				bm.Init_cmd = SCRIPT_NAME + "/" + e;
			if (f != null && !pwhn.cP.Init_File3in1)
				bm.Agg_cmd = SCRIPT_NAME + "/" + f;

			//insert the contents into the msg, only if out of sync
			//if (!pwhn.cP.synced) {
				bm.Init_content = pwhn.cP.Init_ScriptContent;
				if (!pwhn.cP.Init_File3in1)
					bm.Agg_content = pwhn.cP.Upd_ScriptContent;
		/*	
		} else { //else insert only the hashes
				//make both hashes and insert
				bm.Init_content = (byte[]) new utep.plmr.CmdExecer(
						HASHER_SCRIPT + " " + pwhn.cP.Init_File,
						"\n".getBytes(),
						null).execute();
				if (f != null && !pwhn.cP.Init_File3in1)
					bm.Agg_content = (byte[]) new utep.plmr.CmdExecer(
							HASHER_SCRIPT + " " + pwhn.cP.Upd_File,
							"\n".getBytes(),
							null).execute();
				else
					bm.Agg_content = new byte[0];

			}
*/
			Socket server = null;
			try {
				//open connection to first IP
				System.out.println("Connecting to " + IPs[0] + ":"
						+ plmr_server_main.PLMR_PORT + " on socket " + server);
				server = new Socket(IPs[0], plmr_server_main.PLMR_PORT);
				//serialize the msg
				ObjectOutputStream ois = new ObjectOutputStream(server
						.getOutputStream());

				while (!server.isConnected())
					System.out.println("waiting for connection..");

				//	and write off
				if (!server.isOutputShutdown()) {
					//System.out.println(ois + " output open ");
					System.out.println(ois + " writing " + bm);
					ois.writeObject(bm);
				}
			} catch (Exception ex) {
				ex.printStackTrace();
				return;
			}

			ResultMessage rm = null;
			int tries = 0;

			int tot_read = 0;
			ByteBuffer bb = null, bb2 = ByteBuffer.allocateDirect(0);

			try {
				InputStream is = server.getInputStream();

				while (is.available() < 256)
					Thread.sleep(100);

				System.out.println(server
						+ " channel has data available, reading...");

				//check if message is there by trying to parse into bm
				//try a couple times, sleep, then abort
				//and try to construct a resmessage from the stream

				while (tries < MAX_DESER_TRIES && rm == null) {
					int slept = 0;
					while ((is.available() < 1) && (slept < 1000)) {
						slept++;
						Thread.sleep(10);
					}
					System.out.println("slept4:" + slept);

					//Thread.sleep(1); //just preempt
					tries++;

					byte[] tempbuf = new byte[is.available()];
					int read = is.read(tempbuf);

					//implies that the channel is closed
					if (read == -1) {
						throw new IOException(
								"Error on read - the channel has been closed.");
					}

					//System.out.println(this + " read " + read);
					tot_read += read;
					//System.out.println(this + "total read " + tot_read);

					//make new array with total read
					byte[] b = new byte[tot_read];
					bb = ByteBuffer.allocateDirect(tot_read);

					//get the old data
					bb2.rewind();
					bb.put(bb2);

					//and the new
					bb.put(tempbuf);

					bb.rewind();
					//copy into byte array
					bb.get(b);
					//System.out.println("Contents of byte-array:"+new String(b));
					ByteArrayInputStream bais = new ByteArrayInputStream(b);

					Object o = null;
					try {
						ObjectInputStream ois = new ObjectInputStream(bais);
						o = ois.readObject();
					} catch (Exception ice) {
						ice.printStackTrace();
						System.out
								.println("deser. failed reading " + tot_read
										+ " bytes on " + tries
										+ "th try, continuing..");
						bb2 = bb;
						continue;
					}

					if (o instanceof ResultMessage) {
						rm = (ResultMessage) o;
						System.out.println(this + " read "
								+ ((ResultMessage) o).toFullString());

					} else if (o instanceof RSyncReqMessage) {
						System.out.println(this + " read "
								+ ((RSyncReqMessage) o).toFullString());
						processRsyncMsg((RSyncReqMessage) o, server);
						tries = tot_read = 0;
						bb = null;
						bb2 = ByteBuffer.allocateDirect(0);

					}
				}
			} catch (Exception ex) {
				ex.printStackTrace();
				try {
					server.close();
				} catch (Exception exc) {
					exc.printStackTrace();
				}
			}

			//we got something, yeah
			if (rm != null) {
				pwhn.cP.synced = true;

				if (RunName == null)
					RunName = DateFormat.getDateTimeInstance().format(
							new Date()).replace(',', ';');

				//check if we are to evaluate
				byte[] res = null;

					
				if (!pwhn.cP.Init_File3in1){
					if (g != null) {
					//run the eval'er from the native runner
						try {
							res = new FlatNativeRunner().eval(rm.result, g);
						} catch (Exception ex) {
							ex.printStackTrace();
							pwhn.stop();
							return;
						}
					} else
						res = rm.result;
				}else{
					String cmd_l = " "+Runners.S3in1_cmdlines[2];
					try {
						res = new FlatNativeRunner().eval(rm.result, f+cmd_l);
					} catch (Exception ex) {
						ex.printStackTrace();
						pwhn.stop();
						return;
					}
				}
			

				//and add the run
				pwhn.cP.Res_Content.addRun(
						RunName,
						res,
						System.currentTimeMillis() - start,
						type,
						IPs.length,
						rm.path.numNodes,
						rm.path,
						rm.result.length);

			}
			pwhn.stop();
		}//end run

		/**
		 * Processes and reacts to an RSyncRequest msg by initiating an rsync
		 * @param rsm
		 * 			The Msg to get Infos from
		 * @param s
		 * 			the socket that this msg came over, to send the rsyncsentmsg to
		 * @throws IOException
		 */
		private void processRsyncMsg(RSyncReqMessage rsm, Socket s)
				throws IOException {
			String f;

			if (pwhn.cP.Init_FileIsDir)
				f = pwhn.cP.Init_File;
			else {
				f = rsm.ReqedFile;
				if (f.startsWith(SCRIPT_NAME + "/"))
					f = f.substring(5);

				if (f.equals(new File(pwhn.cP.Init_File).getName()))
					f = pwhn.cP.Init_File;
				else if (f.equals(new File(pwhn.cP.Upd_File).getName()))
					f = pwhn.cP.Upd_File;
			}
			String cmd = "rsync -arz " + f + " " + rsm.username + "@"
					+ rsm.ReqIP + ":~/";

			System.out.println("Rsync req, execing: " + cmd);

			new utep.plmr.CmdExecer(cmd, null, null).execute();

			RSyncSentMessage rssm = new RSyncSentMessage(
					null,
					rsm.from,
					rsm.query_ID,
					rsm.ReqedFile);

			ObjectOutputStream ois = new ObjectOutputStream(s.getOutputStream());
			ois.writeObject(rssm);
			System.out.println("sent " + rssm.toFullString());

		}//end process

	}//and end treeRunner

}//end runners
