package utep.plmr;

import java.util.Hashtable;

import utep.plmr.server.PathInfo;

/**
 * Class of Visitors that implement the Visitor IFace for visiting PI nodes
 * @author derDoc
 *
 */
public class Visitors {

	/**
	 * Visitor for diplay of delay vs. size histo
	 * @author derDoc
	 *
	 */
	public class DelayHistoVisitor implements PIVisitor {

		public Hashtable		ht	= new Hashtable();

		public double		max	= 0;

		/**
		 * visits a node
		 * @param p
		 * 			the node
		 * @param level
		 * 			level of node in tree
		 */
		public void visitPI(PathInfo p, int level) {

			//only if the delay isnt <0
			if (p.delay2parent >= 0) {
				//get the val for the key
				Double del = new Double(p.delay2parent);
				if (p.delay2parent > max)
					max = p.delay2parent;
				//test if already there
				double i = 0;
				if (ht.containsKey(del))
					i = ((Double) ht.get(del)).doubleValue();

				//add
				i += p.payload_size;
				//and put in HT
				ht.put(del, new Double(i));

			}
		}
	}
	
	/**
	 * Visitor for building of stats about a run
	 * @author derDoc
	 *
	 */
	public class StatsBuilderVisitor implements PIVisitor {

		//averages are actually running totals, that are divided by their
		//*_num pendants when toString'ing
		public long			avg_b_link		= 0;

		public int			link_num			= 0;
		
		public int			msec_num			= 0;
		
		public float			avg_fan_in		= 0;

		public int			int_nodes_num	= 0;
		
		public float			avg_path_len		= 0;

		public int			path_num			= 0;

		/**
		 * visit a node
		 * @param p
		 * 		the node
		 * @param level
		 * 		node's level
		 */
		public void visitPI(PathInfo p, int level) {
			
			//for every node
			this.avg_b_link += p.payload_size;
			this.link_num++;
			
			this.msec_num += p.delay2parent;
		
			//avg fan-in only if not leaf
			if (p.children.length > 0){
				this.avg_fan_in += p.children.length;
				this.int_nodes_num++;
			}else{	//path len only if leaf
				this.avg_path_len += level;
				this.path_num++;
			}
		}
		
		/**
		 * get infos in this instance as a string
		 * @return the Stats as String
		 */
		public String toString(){
			return
				"Avg. bytes/link:\t" + ((float)avg_b_link/link_num) +"\n" +
				"Avg. bytes/msec:\t" + ( (msec_num>0) ? new Float(avg_b_link/msec_num).toString():"n/a" ) + "\n" +
				"Avg. link delay:\t" + ((float)msec_num/link_num) 	 +"\n" +
				"Avg. fan-in:\t\t"	 + ((float)avg_fan_in/int_nodes_num)+"\n" +
				"Avg. path length:\t"+ ((float)avg_path_len/path_num)+"\n";
		}
		
	}
}
