package utep.plmr.jfree;

import org.jfree.data.Range;
import org.jfree.data.xy.*;

/**
 * Class for Holding a collection of XYZ Series Items
 * @author derDoc
 *
 */
public class XYZSeriesCollection extends XYSeriesCollection implements XYZDataset {

	/**
	 * Consturctor
	 *
	 */
	public XYZSeriesCollection(){
		this(null);
	}
	
	/**
	 * New Collection with Series in it
	 * @param xs
	 */
	public XYZSeriesCollection(XYZSeries xs){
		super(xs);
	}
	
	/**
	 * Get a Z val
	 * @param series
	 * 			the series to get Z from
	 * @param
	 * 			the item
	 * @return
	 * 			the Z val
	 */
	public Number getZ(int series, int item) {
        XYZSeries ts = (XYZSeries) this.getSeries(series);
        return ts.getZ(item);
    }

	/**
	 * Same as above but get the Z val as double
	 */
	public double getZValue(int series, int item) {
        XYZSeries ts = (XYZSeries) this.getSeries(series);
        return ts.getZ(item).doubleValue();
    }
	
	/**
	 * Find Z bounds of all Series in Collection
	 * @return
	 * 			The Range
	 */
	public Range findZBounds(){
		double lower = Double.MAX_VALUE;
		double upper = Double.MIN_VALUE;
		for (int i = 0; i < super.getSeriesCount(); i++){
			Range r = ((XYZSeries)super.getSeries(i)).findZBounds();
			
			if ( r.getLowerBound() < lower)
				lower = r.getLowerBound();
			if( r.getUpperBound() > upper)
				upper = r.getUpperBound();
		}
		return new Range(lower,upper);
	}
}
