package utep.plmr.jfree;

import java.awt.*;
import java.awt.geom.Rectangle2D;

import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.entity.EntityCollection;
import org.jfree.chart.plot.*;
import org.jfree.chart.renderer.xy.*;
import org.jfree.data.xy.*;
import org.jfree.data.*;


/**
 * A Custom Linve&Shape renderer Component, that can set the color per item from its Z-val
 * @author derDoc
 *
 */
public class myLASRenderer extends XYLineAndShapeRenderer {

	// values in 0..360 for H of color
	public static int LowerColorBound = 120;	//green
	public static int UpperColorBound = 0;		//red
	public static int ColorRange = 120;

	// direction around wheel e [1,-1] from lower to upper
	public int direction = -1;

	private double LowerDatasetBound;
	private double UpperDatasetBound;
	private boolean DomainBoundsInitialised = false;

	/**
	 * Creates this LASRenderer
	 * @param lines
	 * 			draw lines
	 * @param shapes
	 * 			draw shapes
	 */
	public myLASRenderer(boolean lines, boolean shapes) {
		super(lines, shapes);
	}

	/**
	 * override draw2nd pass from super class to be able to set color per item
	 * Sets the color from the items Z-val
	 * gets domain bounds and spreads vals over available color range
	 */
	protected void drawSecondaryPass(Graphics2D g2, XYPlot plot,
			XYDataset dataset, int pass, int series, int item,
			ValueAxis domainAxis, Rectangle2D dataArea, ValueAxis rangeAxis,
			CrosshairState crosshairState, EntityCollection entities) {
		
		//if no Z value call super
		if (!(dataset instanceof XYZDataset))
			super.drawSecondaryPass(g2, plot, dataset, pass, series, item,
					domainAxis, dataArea, rangeAxis, crosshairState, entities);
		else {
			XYZDataset xyz = (XYZDataset) dataset;
			//get the bounds of the Z domain
			if (!DomainBoundsInitialised) {
				Range r = ((XYZSeriesCollection) xyz).findZBounds();
				LowerDatasetBound = r.getLowerBound();
				UpperDatasetBound = r.getUpperBound();
				DomainBoundsInitialised = true;
			}
			double z = xyz.getZValue(series, item);

			// first find range in 0..1 of z value between bounds
			float p = ((float) (z - LowerDatasetBound) / (float) (UpperDatasetBound - LowerDatasetBound));

			// multiply by color range
			p = p * ColorRange;

			// and add to base color, heeding direction 
			p = (LowerColorBound + (p * direction));

			//clip to 0..360
			if (p < 0)
				p = 360 + p;
			else if (p > 360)
				p -= 360;

			// HSB needs Hue in 0..1, so divide by 360, sat=1, bright=1
			Paint paint = Color.getHSBColor(p / 360, 1, 1);

			// set color as color for this shape, saving old..
			Paint old;
			if (super.getUseFillPaint()){
				old = super.getSeriesFillPaint(series);
				super.setSeriesFillPaint(series, paint, false);
			}else{
				old = super.getSeriesPaint(series);
				super.setSeriesPaint(series, paint, false);
			}
			//..draw..
			super.drawSecondaryPass(g2, plot, dataset, pass, series, item,
					domainAxis, dataArea, rangeAxis, crosshairState, entities);

			//..and reset old paint
			if (super.getUseFillPaint()){
				super.setSeriesFillPaint(series, old, false);
			}else{
				super.setSeriesPaint(series, old, false);
			}
			
		}
	}

	/**
	 * Inits this instance
	 * Just sets the boundsInit to false
	 */
	public XYItemRendererState initialise(Graphics2D g2, Rectangle2D dataArea,
			XYPlot plot, XYDataset data, PlotRenderingInfo info) {
		DomainBoundsInitialised = false;
		return super.initialise(g2, dataArea, plot, data, info);
	}

}
