package utep.plmr.server;

import java.io.Serializable;
import java.net.*;
import rice.p2p.commonapi.*;

/**
 * PathInfo represents information about the path
 * that a particular run took including all ids, IPs,
 * paths and timings each PI reprs. one node and
 * contains a list of children which it got results from thus it is like a tree
 */
public class PathInfo implements Serializable {

	static final long	serialVersionUID		= 32782l;

	/** IP,hostname of this node */
	public InetAddress	node_IP;

	/** FP ID of node */
	public Id			node_id;

	/**number of items in path, for easy displaying... */
	public int			numNodes;

	/**lat of node, so that client does not have to ask dlab for everything */
	public String		lat;

	public String		lon;

	
	/**
	 * @uml.property  name="children"
	 * @uml.associationEnd  multiplicity="(0 -1)"
	 *  children, that this node got results from, null if leaf
	 */
	public PathInfo[]	children;

	/** delay to parent (filled in by parent), null if root
	*uses TS_sent - not very exact I know...but what the heck*/
	public long			delay2parent;

	/**serialized length in bytes of the ResMsg that contained this PathInfo,
	*ie. size of payload (result) send over link with roughly above latency*/
	public long			payload_size;

	/**time this was sent to parent (as Unix TS)*/
	public long			TS_sent;
	
	/**perceived size of ring, i.e. number of joins vs. leaves seen*/
	public int			ringSize;

	public PathInfo() {}

	/**
	 * constructor
	 * @param isa
	 * 			InetAddress of this node
	 * @param ni
	 * 			Its FP ID
	 * @param c
	 * 			PI[] of Children
	 * @param sent
	 * 			Time this was created
	 */
	public PathInfo(InetAddress isa, Id ni, PathInfo[] c, long sent) {
		this.node_IP = isa;
		this.node_id = ni;
		this.children = c;
		this.TS_sent = sent;
	}
	
	/**
	 * Same as above but with some more info
	 * @param isa
	 * @param ni
	 * @param c
	 * @param sent
	 * @param nodes
	 * 			number of children
	 * @param lat
	 * 			latitude
	 * @param lon
	 * 			longitude
	 * @param size
	 * 			size of res in bytes
	 * @param delay
	 * 			delay to parent
	 */
	public PathInfo(InetAddress isa, Id ni, PathInfo[] c, long sent,
			int nodes, String lat, String lon, long size, long delay) {
		this(isa,ni,c,sent);
		this.numNodes = nodes;
		this.lat = lat;
		this.lon = lon;
		this.payload_size = size;
		this.delay2parent = delay;
		
	}

	/**
	 * String representation
	 */
	public String toString() {
		String r =  "PathInfo{\n" + 
			"\tIP:\t" + node_IP + "\n" + 
			"\tId:\t" + node_id + "\n" + 
			"\t#Nodes:\t" + numNodes + "\n" + 
			"\tlat:\t" + lat + "\n" + 
			"\tlon:\t" + lon + "\n" + 
			"\tsize:\t" + payload_size + "\n" + 
			"\tDelay:\t" 	+ delay2parent + "\n" + 
			"\tsent:\t" + TS_sent + "\n" +
			"\t#Ring:\t" + ringSize + "\n" +
			"\tChildren:{\n";
			
		for (int i=0; i<children.length;i++)
			r += children[i];
		
		r += "\t\t}\n}\n";
		
		return r;
	}

}
