package utep.plmr.server.messages;

import rice.p2p.commonapi.*;
import rice.pastry.Id.Distance;
import rice.pastry.routing.RouteSet;
import rice.pastry.socket.SocketNodeHandle;

import utep.plmr.server.IdHelper;

/**
 * Class that repr. a Build Tree Req.
 * @author derDoc
 *
 */
public class BuildMessage implements Message {

	// defs for types of trees
	
	public transient static final int	FP_FTT				= 2;

	public transient static final int	FP_KBT				= 3;

	public transient static final int	CO_KBT				= 4;

	/** serial UID for serialization*/
	static final long					serialVersionUID		= 32780l;

	// stuff for FP
	/**
	 * Where the Message came from.
	 */
	public Id							from;

	/**
	 * Where the Message is going.
	 */
	public Id							to;
	
	
	/**
	 * Where should have been coming from.
	 */
	public Id							senderProxyFor;
	
	
	/** actual content of msg (reminder: purpose of msg is to build a tree) * */

	/** type of tree requested */
	public int							tree_type;

	/** the root of tree */
	public Id							root;

	/**the unique ID of this query per root, used to identify the query at each node
	/* this is the key into a HT for all outstanding queries,
	/* normally just hostname + UnixTS  */
	public String						query_ID;

	/** List of IPs to include result from  */
	public String[]						IPList;

	/** things to execute * */

	/** if init is a local cmd, this is its path */
	public String						Init_cmd;

	/** else if its included here, this is it */
	public byte[]						Init_content;

	/** if agg is a local cmd, this is its path */
	public String						Agg_cmd;

	/** else if its included here, this is it */
	public byte[]						Agg_content;
	
	/**and the rsync base if it is a dir */
	public String						RSyncBaseDir;
	
	/**wheter we are collecting path Info data */
	public boolean						bCollPathInfo;
	
	/**whether its a 3-in-1 file */
	public boolean						bFile3in1;
	
	/**Timeout for this level of tree */
	public long							timeout;
	

	public BuildMessage() {}

	/**
	 * Constructor.
	 * 
	 * @param from
	 *            DESCRIBE THE PARAMETER
	 * @param to
	 *            DESCRIBE THE PARAMETER
	 */
	public BuildMessage(Id from, Id to) {
		this.from = from;
		this.to = to;
	}

	/**
	 * Use low priority to prevent interference with overlay maintenance
	 * traffic.
	 * 
	 * @return The Priority value
	 */
	public byte getPriority() {
		return Message.LOW_PRIORITY;
	}

	public String toString() {
		return new String("BM{" + query_ID + "@" + root + "}");
	}

	/**
	 * retunrs a full string with all variables descr. this instance
	 * 
	 * @return a long string
	 */
	public String toFullString() {
		String r = "BuildMsg{\n" + "\tFrom:\t" + from + "\n" + "\tTo:\t" + to
				+ "\n" + "\tRoot:\t" + root + "\n" + "\tQueryID:\t" + query_ID
				+ "\n" + "\tType:\t" + tree_type + "\n" + "\tIn_path:\t"
				+ Init_cmd + "\n" + "\tIn_cont:\t" + new String(Init_content)
				+ "\n" + "\tAg_path:\t" + Agg_cmd + "\n" + "\tAg_cont:\t"
				+ new String(Agg_content) + "\n" + "\tIPs:{";

		for (int i = 0; i < IPList.length; i++)
			r += IPList[i] + ",";

		r += "\t\t}\n}\n";

		return r;
	}
	
	
	/**
	 * Called by checkQuery in case we need to spread the word down the tree
	 * 
	 * @param node
	 *            global FP node
	 * @param endpoint
	 * 			  global FP endpoint
	 */
	public void sendDown(Node node, Endpoint endpoint) {

		rice.pastry.routing.RoutingTable rt = ((rice.pastry.PastryNode) node)
				.getRoutingTable();
		//		System.out.println("Sending BuildMsg:" + bm + " down");

		Id To = to;
		//proxy=assumed sender, ie. node that should have send msg
		Id Proxy = senderProxyFor;
		Id From = from;

		//insert us as sender into msg
		from = node.getId();

		// handle case where msg comes to us, coz we are closest to non-ex.
		// id, i.e. we need to overtake its role
		// need to check where we are rel. to target and calc. new targets
		// since we are closest, we need to send to all who parent would have sent it to
		// which are farer away than us
		if (to != node.getId()) {
			//have to check if we are closest in XOR distance, before we can accept msg to us

			Distance smallestD = IdHelper.XORDist(
					(rice.pastry.Id) To,
					(rice.pastry.Id) node.getId());
			//the node handle to send to that is closer in XOR, if found
			SocketNodeHandle snh = null;
			System.out.println("Our dist:" + smallestD
					+ ",Original recpt. was " + to);

			int d = ((rice.pastry.Id) node.getId()).indexOfMSDD(
					(rice.pastry.Id) To,
					4);
			//int myCol = ((rice.pastry.Id) node.getId()).getDigit(d, 4);

			//for that we need to check each digit starting from the one that differs to the end
			for (int i = d; i >= 0; i--) {
				//and all 16 cols per digit
				for (int j = 0; j < 16; j++) {

					RouteSet rs = rt.getRouteSet(i, j);
					if (rs == null || rs.size() == 0)
						continue;

					SocketNodeHandle nh = (SocketNodeHandle) rs.closestNode();
					Distance theirD = IdHelper.XORDist(
							(rice.pastry.Id) To,
							(rice.pastry.Id) nh.getId());

					//if less : remember
					if (IdHelper.XORDistanceComparator(theirD, smallestD) < 0) {
						smallestD = theirD;
						snh = nh;
					}
				}
			}

			//now if there is a closer node, send it directly there, and return
			if (snh != null) {
				System.out.println("Closer node is:" + snh + ",with dist:"
						+ smallestD);
				//to = snh.getId();
				endpoint.route(null, this, snh);
				return;

				//if not, we are the closest node, so act for it
			} else {
				timeout -= 8000;

				//System.out.println("Original recpt. was " + to);
				//get MSDB for parent first
				int f = ((rice.pastry.Id) Proxy)
						.indexOfMSDB((rice.pastry.Id) To) - 1;
				//get our distance to orig. recpt.
				/*Distance ourD = IdHelper.absDist(
				 (rice.pastry.Id) node.getId(),
				 (rice.pastry.Id) To);
				 */
				senderProxyFor = To;

				for (int i = f; i >= 0; i--) {
					//and flip the next bit in To
					Id theirID = IdHelper.flipBit((rice.pastry.Id) To, i);
					//their Dist. to this guy
					//			Distance theirD = IdHelper.absDist(
					//				(rice.pastry.Id) theirID,
					//			(rice.pastry.Id) To);

					//	if (theirD.compareTo(ourD) > 0) {
					to = theirID;

					RouteSet rs = rt.getBestEntry((rice.pastry.Id) to);
					NodeHandle cnh = null, cnh1 = null;

					if (rs != null) {
						cnh = ((SocketNodeHandle) rs.closestNode());

					}

					//and only route if there is a node in that space, which is not the one we got it from, and not us
					if (rs != null && rs.size() > 0 && to != From
							&& to != node.getId() && cnh != null) {

						endpoint.route(to, this, cnh);
						//						System.out.print("Dist. " + theirD + ">" + ourD
						//								+ " sending BM");
						System.out.println("Proxy for:" + senderProxyFor
								+ ",sending BM for " + to
								+ ",over next bit:" + cnh);

					} else {
						//cnh1 = rt.bestAlternateRoute((rice.pastry.Id) theirID);
					}

					if (cnh1 != null && cnh1 != node.getLocalNodeHandle()
							&& cnh1.getId() != From) {

						endpoint.route(theirID, this, cnh1);
						//						System.out.print("Dist. " + theirD + ">" + ourD
						//								+ " sending BM");
						System.out.println("Proxy for:" + senderProxyFor
								+ ",sending BM for " + to + ",over closer:"
								+ cnh);

					}

					//}
				}
			}
		} else
			timeout -= 4000;

		//accepting msg for us
		//and now send to all that are our responsibility

		// get idx of mostSignificantDifferingBit
		int j = ((rice.pastry.Id) Proxy).indexOfMSDB((rice.pastry.Id) node
				.getId());

		// if no diff then we are root, set to MSB (160)
		if (j < 0)
			j = 160;

		senderProxyFor = node.getId();
		
		// and send the msg down the tree, acting for us; start adding the next bit
		for (int i = j - 1; i >= 0; i--) {

			to = IdHelper.flipBit((rice.pastry.Id) node.getId(), i);
			//to = node.getId().addToId(offset);

			RouteSet rs = rt.getBestEntry((rice.pastry.Id) to);
			NodeHandle cnh = null, cnh1 = null;

			if (rs != null) {
				cnh = ((SocketNodeHandle) rs.closestNode());
			}

			//and only route if there is a node in that space, which is not the one we got it from, and not us
			if (rs != null && rs.size() > 0 && to != From
					&& to != node.getId() && cnh != null) {

				//only send if node in that part of the ID space

				endpoint.route(to, this, cnh);
				System.out.println("Sending BM for " + to
						+ ",over next bit:" + cnh);

			} else {
				//cnh1 = rt.bestAlternateRoute((rice.pastry.Id) to);
			}

			if (cnh1 != null && cnh1 != node.getLocalNodeHandle()
					&& cnh1.getId() != From) {

				endpoint.route(to, this, cnh1);
				System.out.println("Sending BM for " + to + ",over closer:"
						+ cnh);

			}
		}
	}

}
