package utep.plmr.server.messages;

import java.text.DateFormat;
import java.util.Date;
import java.io.*;

import rice.p2p.commonapi.*;
import rice.pastry.routing.RouteSet;
import rice.pastry.socket.SocketNodeHandle;

/**
 * Msg to local node to signalize its time for logging the FingerTable
 * @author derDoc
 *
 */
public class LogFTMsg implements Message {

	public byte getPriority() {
		return Message.LOW_PRIORITY;
	}

	/**
	 * Process this msg and Log to the File
	 * @param node
	 * 			Global Node for RoutingTable
	 * @param LogFile
	 * 			the Outfile
	 */
	public void process(Node node, BufferedWriter LogFile) {
		
		rice.pastry.routing.RoutingTable rt = ((rice.pastry.PastryNode) node)
				.getRoutingTable();

		try {
			//just log the FT
			LogFile.write(DateFormat.getDateTimeInstance().format(new Date())
					+ "(#" + rt.numEntries() + ",(");

			for (int i = 0; i < 40; i++) {

				int myCol = ((rice.pastry.Id) node.getId()).getDigit(i, 4);

				for (int j = 0; j < 16; j++) {
					if (j == myCol)
						continue;

					RouteSet rs = rt.getRouteSet(i, j);
					if (rs == null || rs.size() == 0)
						continue;

					//now for each i in RT print: (i,j,numentries, best entry:(ID,IP,live))
					LogFile.write("(" + i + "," + j + "," + rs.size() + ",(");
					SocketNodeHandle nh = (SocketNodeHandle) rs.closestNode();
					LogFile.write(nh.getId() + "," + nh.getAddress() + ","
							+ nh.proximity() + ")),");
				}

			}
			LogFile.write("))");
			LogFile.newLine();
			LogFile.flush();

		} catch (Exception e) {
			e.printStackTrace();
		}
	}

}
