package utep.plmr.server.messages;

import rice.Continuation;
import rice.p2p.commonapi.*;
import java.util.*;

import utep.plmr.server.Query;

/**
 * Msg to let the recipient know that we need an rsync
 * @author derDoc
 *
 */
public class RSyncReqMessage implements Message {

	/**
	 * Where the Message came from.
	 */
	public Id		from;

	/**
	 * Where the Message is going.
	 */
	public Id		to;

	/** IP of requester*/
	public String	ReqIP;

	/** file requested*/
	public String	ReqedFile;

	/**ID of query that this req pertains to*/
	public String	query_ID;

	/**username for logging in with SSH*/
	public String	username	= "";

	/**
	 * Constructor
	 * @param from
	 * 			ID from
	 * @param to
	 * 			ID to
	 * @param i
	 * 			local IP for req
	 * @param f
	 * 			File
	 */
	public RSyncReqMessage(Id from, Id to, String i, String f) {
		this.from = from;
		this.to = to;
		this.ReqIP = i;
		this.ReqedFile = f;
	}

	public byte getPriority() {
		return Message.LOW_PRIORITY;
	}

	public String toString() {
		return "RSyncReqMsg{" + query_ID + ":" + ReqedFile + "@" + ReqIP + "}";
	}

	public String toFullString() {
		return "RSyncReqMsg{\n" + "From:" + from + "\n" + "To:" + to + "\n"
				+ "File:" + ReqedFile + "\n" + "4ID:" + query_ID + "\n"
				+ "SendTo:" + ReqIP + "\n" + "user:" + username + "\n}";
	}

	/**
	 * Process this msg
	 * @param node
	 * 			global FP node
	 * @param endpoint
	 * 			global FP endpoint
	 * @param queries
	 * 			outstanding Queries table
	 */
	public void process(final Node node, final Endpoint endpoint, Hashtable queries) {
		//rsync requested, check if we have the newest version
		Query q = (Query) queries.get(query_ID);
		if (q != null && q.bAwaitRsync) {
			//no, stick in queue
			q.childRsyncs.add(this);
		} else {
			//si, sched rsync
			String cmd = "rsync -arz " + ReqedFile + " " + username
					+ "@" + ReqIP + ":~/";

			System.out.println("Rsync req, execing: " + cmd);

			((rice.pastry.PastryNode) node).process(new utep.plmr.CmdExecer(
					cmd,
					null,
					null), new Continuation() {
				// will be called if success of the exec
				public void receiveResult(Object res) {
					//send the success msg
					RSyncSentMessage rssm = new RSyncSentMessage(
							node.getId(),
							from,
							query_ID,
							ReqedFile);
					endpoint.route(from, rssm, null);
				}

				// will be called if failure
				public void receiveException(Exception result) {}
			});
		}
	}
}
