package utep.plmr.server.messages;

import java.io.File;
import java.util.*;

import rice.Continuation;
import rice.p2p.commonapi.*;
import utep.plmr.Runners;
import utep.plmr.server.Query;

/** 
 * Msg to let recpt. know that RSync was sent
 * @author derDoc
 *
 */
public class RSyncSentMessage implements Message {

	
	/**
	 * Where the Message came from.
	 */
	public Id					from;

	/**
	 * Where the Message is going.
	 */
	public Id					to;

	/** ID of query that the rsynreq msg was given*/
	public String				query_ID;

	/** Path of synced file just to make sure*/
	public String				SyncedFile;

	/** Constr.
	 * 
	 * @param from
	 * @param to
	 * @param q
	 * @param s
	 */
	public RSyncSentMessage(Id from, Id to, String q, String s) {
		this.from = from;
		this.to = to;
		this.query_ID = q;
		this.SyncedFile = s;
	}

	public byte getPriority() {
		return Message.LOW_PRIORITY;
	}

	public String toString() {
		return "RSyncSentMsg{" + query_ID + "}";
	}

	public String toFullString() {
		return "RSyncSentMsg{\n" + "From:" + from + "\n" + "To:" + to + "\n"
				+ "4ID:" + query_ID + "\n" + "\n}";
	}

	/**
	 * called on this Msg to process the Request
	 * @param node
	 * 			global FP node
	 * @param endpoint
	 * 			global FP endpoint
	 * @param queries
	 * 			Queries Hashtable
	 * @param ssh_username
	 * 			Username for SSH
	 */
	public void process(final Node node, final Endpoint endpoint,
			Hashtable queries, String ssh_username) {

		//rsync requested, check if we have the newest version
		final Query q = (Query) queries.get(query_ID);
		//if query already went away coz it took to long, forget
		if (q == null)
			return;
		q.bAwaitRsync = false;

		System.out.println("Got Rsync for: " + SyncedFile);

		//delete the old hash
		new File(SyncedFile + ".sha1").delete();
		//and make a new
		((rice.pastry.PastryNode) node).process(new utep.plmr.CmdExecer(
				utep.plmr.server.plmr_srv_app.HASHER_SCRIPT + " " + SyncedFile,
				"\n".getBytes(),
				null), new Continuation() {
			public void receiveResult(Object res) {}

			public void receiveException(Exception result) {}
		});

		//if is waiting, sched an init
		if (q.bAwaitInit_cmd != null) {
			System.out.println("Sched'ing init run of: " + q.bAwaitInit_cmd);
			String cmd_l = "";
			if (q.bFile3in1)
				cmd_l = " " + Runners.S3in1_cmdlines[0];

			((rice.pastry.PastryNode) node).process(new utep.plmr.CmdExecer(
					q.bAwaitInit_cmd + cmd_l,
					null,
					null), new Continuation() {
				// will be called if success of the exec
				public void receiveResult(Object result) {
					try {
						q.addResult((byte[]) result);
						//and delete if temp file
						if (q.bAwaitInit_cmd != null
								&& q.bAwaitInit_cmd
										.matches("^.*?/Init-(\\d){10,}$")) {
							new File(q.bAwaitInit_cmd).delete();
						}
					} catch (Exception exc) {
						exc.printStackTrace();
					}
				}

				// will be called if failure
				public void receiveException(Exception result) {
					System.out.println("There was an error in Init: " + result);
				}
			});
			q.bAwaitInit_cmd = null;
		}

		//and rsync child rsync reqs if any
		while (q.childRsyncs.size() > 0) {
			//get rsm
			final RSyncReqMessage rsrm = (RSyncReqMessage) q.childRsyncs
					.remove(0);

			String cmd = "rsync -arz " + rsrm.ReqedFile + " " + ssh_username
					+ "@" + rsrm.ReqIP + ":~/";
			System.out.println("Sched'ing rsync to waiting child: " + cmd);

			((rice.pastry.PastryNode) node).process(new utep.plmr.CmdExecer(
					cmd,
					null,
					null), new Continuation() {
				// will be called if success of the exec
				public void receiveResult(Object res) {
					//send the success msg
					RSyncSentMessage rssm = new RSyncSentMessage(
							node.getId(),
							rsrm.from,
							rsrm.query_ID,
							rsrm.ReqedFile);
					endpoint.route(rsrm.from, rssm, null);
				}

				// will be called if failure
				public void receiveException(Exception result) {}
			});
		}

	}

}
