package utep.plmr.server.messages;

import rice.p2p.commonapi.*;
import rice.p2p.commonapi.Message;
import utep.plmr.server.*;
import java.util.*;

/**
 * Msg containing results of subtree
 * Contains contents and PI
 * @author  derDoc
 */
public class ResultMessage implements Message {

	/**
	 * Where the Message came from.
	 */
	public Id		from;

	/**
	 * Where the Message is going.
	 */
	public Id		to;

	//content of msg

	/**	the unique ID of this query per root, used to identify the query at each node*/
	public String	query_ID;

	/**the result*/
	public byte[]	result	= new byte[0];

	/**and info about path of result*/
	public PathInfo	path;

	/**the root for everybody who doesnt know that query*/
	public Id		root;

	/**
	 * Empty Constructor.
	 */
	public ResultMessage() {}

	/**
	 * Constructor.
	 *
	 * @param from DESCRIBE THE PARAMETER
	 * @param to DESCRIBE THE PARAMETER
	 */
	public ResultMessage(Id from, Id to) {
		this.from = from;
		this.to = to;
	}

	/**
	 * Use low priority to prevent interference with overlay maintenance traffic.
	 *
	 * @return The Priority value
	 */
	public byte getPriority() {
		return Message.LOW_PRIORITY;
	}

	public String toString() {
		return "ResMsg{" + query_ID + "@" + root + "}";
	}

	/**
	 * returns a full describing string
	 *
	 * @return string comprising all values
	 */
	public String toFullString() {
		if (result == null)
			result = new byte[0];

		return "ResMsg{\n" + "\tFrom:\t" + from + "\n" + "\tTo:\t" + to + "\n"
				+ "\tq_id:\t" + query_ID + "\n" + "\troot:\t" + root + "\n"
				+ "\tRes:\t" + new String(result) + "\n" + "\tPath:\t" + path
				+ "\n" + "}\n";
	}

	/**
	 * Process this msg
	 * @param queries
	 * 			Queries table
	 * @param endpoint
	 * 			global FP endpoint
	 * @param node
	 * 			global FP node
	 */
	public void process(Hashtable queries, Endpoint endpoint, Node node) {
		// calc. delay
		path.delay2parent = (System.currentTimeMillis() - path.TS_sent);

		Query q = (Query) queries.get(query_ID);

		// if not known, send towards root and hope somebody knows it!
		if (q == null) {
			System.out
					.println("Got result for unknown query, routing towards root:"
							+ root);
			if (root != node.getId())
				endpoint.route(root, this, null);
			else
				System.out.println("We are the root, dropping !");
			return;
		}

		// and add result
		q.addResult(result);
		// also add pathInfo of that msg to q
		q.paths.add(path);
		q.numNodes += path.numNodes;

		// check if aggregation should be scheduled
		q.checkAndAgg(node);
	}
}
