package utep.plmr.server.messages;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.net.InetAddress;

import rice.p2p.commonapi.*;

import utep.plmr.server.PathInfo;
import utep.plmr.server.Query;

/**
 * Msg to self to let node know that it is time to sent the results up the tree
 * @author  derDoc
 */
public class SendResMsg implements Message {

	/** Query to send */
	public Query q;
	
	/** Already tried to send but was canceled due to errors? */
	public boolean bSentOnce = false;
	
	public SendResMsg(Query qu, boolean s){
		this.q = qu;
		this.bSentOnce = s;
	}
	
	public byte getPriority() {
		return Message.LOW_PRIORITY;
	}

	public String toString() {
		return "SendResMsg{"+ q + "}";
	}
	
	/**
	 * Called on this msg to send the result up the tree
	 * @param endpoint
	 * 			global FP endpoint
	 * @param node
	 * 			global FP node
	 * @param perc_RingSize
	 * 			global FP perceived Ring Size
	 * @param lat
	 * 			latitude of this node
	 * @param lon
	 * 			Longitude of this node
	 */
	public void process(Endpoint endpoint, Node node,int perc_RingSize,String lat,String lon){
//		 msg to ourselves that it is about time to send a res msg
		// check if there are any outstanding agg to do
		int to = q.checkAndAgg(node);

		//if TO=5000 and bSentOnce we already waited long enough, just send
		if (to > 0 && (to < 5000 || !bSentOnce))
			// need to wait a little, resched
			((rice.pastry.commonapi.PastryEndpoint) endpoint)
					.scheduleMessage(new SendResMsg(q, true), to);

		else { // no, send the res

			ResultMessage rm = new ResultMessage();
			rm.from = node.getId();
			if (q.results.size() > 0) {
				//concatenate in case there is > 1
				ByteArrayOutputStream out = new ByteArrayOutputStream();
				byte[] ls;
				try {
					while (q.results.size() > 0) {
						ls = (byte[]) q.results.remove(0);
						out.write(ls);
						out.write('\n');
					}
				} catch (IOException ioex) {
					ioex.printStackTrace();
				}

				rm.result = out.toByteArray();

			} else
				rm.result = new byte[0];

			rm.query_ID = q.query_ID;
			rm.root = q.root;
			//only insert path info if requested
			if (q.bCollPathInfo) {
				try {
					PathInfo pi = new PathInfo(
							InetAddress.getLocalHost(),
							node.getId(),
							(PathInfo[]) q.paths.toArray(new PathInfo[0]),
							System.currentTimeMillis());
					pi.payload_size = rm.result.length;
					pi.lat = lat;
					pi.lon = lon;
					pi.numNodes = q.numNodes;
					pi.ringSize = perc_RingSize;
					rm.path = pi;

				} catch (Exception e) {
					e.printStackTrace();
				}
			}

			// see if we are the last node, ie. theres a client_conn
			if (q.client != null) { // yes, send to client, and close
				//check if we need to eval
				System.out.println("Sending result to client");
				q.client.send(rm);
				q.client.close();

			} else { // no, send to next node
				Id sendTo = q.getNextParent();

				System.out.println("Sending result to node: " + sendTo);
				rm.to = sendTo;
				// and queue for sending if ID
				if (sendTo != null)
					endpoint.route(sendTo, rm, null);

			}
			// and remove query from HT
			//delete temp agg file
			if (q.Agg_cmd.matches("^.*?/Agg-(\\d){10,}$")) {
				//System.out.println(q.Agg_cmd + " matched");
				new File(q.Agg_cmd).delete();
			}
			//else
			//System.out.println(q.Agg_cmd + " didnt match");

		}
	}
}
