package utep.plmr.server;

import java.io.*;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.SelectionKey;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;
import rice.p2p.commonapi.Message;
import rice.selector.SelectionKeyHandler;
import utep.plmr.server.messages.*;

/** Class for a connection to a client, resp. for serializing and des. msgs to the client
 * @author  derDoc
 */
public class plmr_client_connection extends SelectionKeyHandler {

	public static final int	BUILD_MSG_SIZE	= 1024;

	public static final int	MAX_DESER_TRIES	= 100;

	/** the key to read from*/
	private SelectionKey		key;

	private SocketChannel	chan;

	/** the buffer used to read the header*/
	private ByteBuffer		buffer;

	//private byte[]					array			= null;

	private plmr_srv_app				parent;
	/** count tries for giving up*/
	private int tries = 0;
	/** total read for resizing of buffer after each read*/
	private int tot_read = 0;
	

	/**
	 * constructor
	 * @param key
	 * 			the selection key to be notified if something happens
	 * @param plmr
	 * 			parent application to call
	 */
	public plmr_client_connection(SelectionKey key, plmr_srv_app plmr)
			throws IOException {
		this.buffer = ByteBuffer.allocateDirect(BUILD_MSG_SIZE);
		this.parent = plmr;
		acceptConnection(key);
	}

	/**
	 * Method which closes down this socket manager, by closing the socket,
	 * cancelling the key and setting the key to be interested in nothing
	 */
	public void close() {
		System.out.println(this + " closing");

		try {
			if (key != null) {
				key.channel().close();
				key.cancel();
				key.attach(null);
				key = null;
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Accepts a new connection on the given key
	 * 
	 * @param serverKey
	 *            The server socket key
	 * @exception IOException
	 *                DESCRIBE THE EXCEPTION
	 */
	protected void acceptConnection(SelectionKey serverKey) throws IOException {
		final SocketChannel channel = (SocketChannel) ((ServerSocketChannel) serverKey
				.channel()).accept();
		System.out.println(this + " accepting connx from " + channel);
		this.chan = channel;

		channel.socket().setSendBufferSize(plmr_srv_app.SOCKET_BUFFER_SIZE);
		channel.socket().setReceiveBufferSize(plmr_srv_app.SOCKET_BUFFER_SIZE);
		channel.configureBlocking(false);

		this.key = parent.node.getEnvironment().getSelectorManager().register(
				channel,
				this,
				SelectionKey.OP_READ);

	}

	/**
	 * Reads from the socket attached to this connector.
	 * 
	 * @param key
	 *            The selection key for this manager
	 */
	public void read(SelectionKey key) {

		int oldPos = 0;
		//System.out.println(this+" has data avail., reading...");
		
		try {
			Message m = null;

			while (tries < MAX_DESER_TRIES && m == null) {
				Thread.sleep(1); //preempt
				tries++;
				
				//change size of buffer, if full
				if (buffer.position() == buffer.capacity()){
					ByteBuffer bb2 = ByteBuffer.allocateDirect(buffer.capacity()*2);
					buffer.position(0);
					bb2.put(buffer);
					buffer = bb2;
				}
				int read = ((SocketChannel) key.channel()).read(buffer);
				
				//System.out.println(this + " read " + read);
				tot_read += read;
				//System.out.println(this + "total read " + tot_read);

				// implies that the channel is closed
				if (read == -1) {
					throw new IOException(
							"Error on read - the channel has been closed.");
				}
				/* check if message is there by trying to parse into bm
				* try a couple times, sleep, then abort
				* and try to construct a buildmessage from the stream
				*/
				
				//make fresh byte array
				byte[] b = new byte[tot_read];
				//remember old pos, and reset pos of buffer
				oldPos = buffer.position();
				buffer.position(0);
				//copy whole buffer into byte array
				buffer.get(b);
				//and reset old pos for next chunk to be read
				//this function will be callled back, whenever more data is avail. 
				buffer.position(oldPos);

				//try to construct an objInputStream on the byte array
				ByteArrayInputStream bais = new ByteArrayInputStream(b);

				Object o = null;
				//and try to parse
				try {
					ObjectInputStream ois = new ObjectInputStream(bais);
					o = ois.readObject();
				} catch (Exception ice) {
					//System.out.println("deser. failed reading "+tot_read+" bytes on "+tries+" try");
					//ice.printStackTrace();
					//if pasrsing failed, we need to return and wait for more data
					return;
				}
				
				//now we got something, see which obj it is
				if (o instanceof BuildMessage) {
					m = (BuildMessage) o;
					System.out.println(this + " read "
							+ ((BuildMessage) o).toFullString());
					parent.buildTreeHere((BuildMessage)m, this);
					buffer.clear();
					
				}else if (o instanceof RSyncSentMessage) {
					m = (RSyncSentMessage) o;
					//yeah we got hooked up by our client, yehhaa!
					//notify parent
					System.out.println(this + " read "
							+ ((RSyncSentMessage) o).toFullString());
					parent.deliver(null, m);
					buffer.clear();
				}
			}

			
		} catch (Exception e) {
			System.out.println(this+" Error in read loop:");
			e.printStackTrace();
			close();
		}
	}

	/**
	 * Sends a msg through this channel
	 * @param rm
	 * 			the ResMsg to send, but can send any
	 */
	public void send(Message rm) {

		System.out.println(this + " sending " + rm);

		try {
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			ObjectOutputStream ois = new ObjectOutputStream(baos);
			ois.writeObject(rm);

			ByteBuffer bb = ByteBuffer.wrap(baos.toByteArray());

			while (bb.remaining() > 0) {
				((SocketChannel) key.channel()).write(bb);
				Thread.sleep(100);
			}

		} catch (Exception e) {
			close();
		}

	}

	/**
	 * To string
	 */
	public String toString() {
		return new String("ClientConn on: " + this.chan);
	}

}
