package utep.plmr.server;

import rice.environment.Environment;
import rice.pastry.*;
import rice.pastry.socket.SocketPastryNodeFactory;
import rice.pastry.standard.RandomNodeIdFactory;

import java.net.*;

/**
 * Startup class for server, parses cmdline and starts node 
 * @author derDoc
 *
 */
public class plmr_server_main {

	public static final int	FP_PORT		= 31780;

	public static final int	PLMR_PORT	= FP_PORT;
	
	public static final long SLEEP_STEP	= 10000l;
	
	public static final long SLEEP_MAX	= 300000l;	

	/** Static main
	 * @param args
	 */
	public static void main(String[] args) {
		Environment env = new Environment();

		try {
			int bindport = Integer.parseInt(args[0]);

			// build the bootaddress from the command line args
			InetAddress bootaddr = InetAddress.getByName(args[1]);
			int bootport = Integer.parseInt(args[2]);

			int plmrport = Integer.parseInt(args[3]);

			InetSocketAddress bootaddress = new InetSocketAddress(
					bootaddr,
					bootport);

			//System.out.println("Bootadress is " + bootaddress);

			// launch our node!
			new plmr_server_main(bindport, plmrport, bootaddress, env);
		} catch (Exception e) {
			// remind user how to use
			System.out.println("Usage:");
			System.out
					.println("java [-cp FreePastry-<version>.jar] utep.plmr.server.plmr_server_main bindport bootIP bootPort PLMRport");
			System.out
					.println("Example: java utep.plmr.server.plmr_server_main planetlab1.utep.edu 31780");
			e.printStackTrace();
			System.exit(129);
		}
	}

	/**
	 * Constructor 
	 * @param bindport
	 * 				port to bind FP to
	 * @param plmrport
	 * 					port to listen for PWHN clients
	 * @param bootaddress
	 * 					boot address for FP
	 * @param env
	 * 				env for FP
	 * @throws Exception
	 */
	public plmr_server_main(int bindport, int plmrport,
			InetSocketAddress bootaddress, Environment env) throws Exception {

		// Generate the NodeIds Randomly
		NodeIdFactory nidFactory = new RandomNodeIdFactory(env);

		// construct the PastryNodeFactory, this is how we use rice.pastry.socket
		PastryNodeFactory factory = new SocketPastryNodeFactory(
				nidFactory,
				bindport,
				env);

		//System.out.println("Getting nodehandle for " + bootaddress);

		// This will return null if we there is no node at that location
		NodeHandle bootHandle = ((SocketPastryNodeFactory) factory)
				.getNodeHandle(bootaddress);

		//System.out.println("Got " + bootHandle);

		// construct a node, passing the null boothandle on the first loop will cause the node to start its own ring
		PastryNode node = factory.newNode(bootHandle);

		// the node may require sending several messages to fully boot into the ring
		long slept = 0;
		while (!node.isReady()) {
			// delay so we don't busy-wait
			Thread.sleep(SLEEP_STEP);
			slept += SLEEP_STEP;
			System.out.println("Waiting for "+(slept/1000)+"s");
			if (slept > SLEEP_MAX)
				System.exit(1);
				//and exit if we waited longer than 5 mins
		}

		//System.out.println("Finished creating new node " + node);

		// construct a new MyApp
		new plmr_srv_app(node, plmrport);

	}
}
