package utep.plmr.server;

import java.nio.channels.*;
import java.net.*;
import java.io.*;
import java.text.DateFormat;
import java.util.*;

import rice.Continuation;
import rice.environment.params.Parameters;
import rice.p2p.commonapi.*;
import rice.p2p.commonapi.NodeHandle;
import rice.selector.*;
import rice.pastry.Id.Distance;
//import rice.pastry.*;
import rice.pastry.routing.*;
import rice.pastry.socket.*;

import utep.plmr.*;
import utep.plmr.server.messages.*;

/**
 * Main class of server, impl. all necc. functions for Appl. for FP
 * @author derDoc
 *
 */
public class plmr_srv_app implements Application {

	/** local node*/
	public Node					node;

	/** the local endpoint to call methods on*/
	protected Endpoint			endpoint;

	/** the key for listening on the PLMR port*/
	//private SelectionKey			key;

	/** hashtable for outstanding queries, maps QueryID -> Query*/
	private Hashtable			queries;

	public static int			SOCKET_BUFFER_SIZE;

	/** time per distance bit in ms to wait*/
	public static final int		WAIT_TIME		= 16;

	public static final int		WAIT_DIV_TIME	= 90000;

	/** time per agg job in ms to wait*/
	public static final int		AGG_WAIT_TIME	= 100;

	/**request string for getting hostname -> (lon,lat) from dlab*/
	public static final String	DLAB_REQ_STRING	= "http://www.robust.cs.utep.edu/~vitus/plmr/allsites.php?hostname=";

	public static final String	HASHER_SCRIPT	= "python PLMR/plmr-server/hasher.py";

	/** lat,lon of node, so that client does not have to ask dlab for everything*/
	String						lat;

	String						lon;

	String						ssh_username;

	/**perceived ring size*/
	int							perc_RingSize	= 0;

	/**log file*/
	BufferedWriter				LogFile;

	//Functions

	/**
	 * Constructor for server_app.
	 * 
	 * @param node
	 *            the Node that has been built
	 */
	public plmr_srv_app(Node node, int plmrport) throws Exception {
		queries = new Hashtable();

		ssh_username = System.getProperty("user.name");
		//System.out.println("username is:"+ssh_username);

		Parameters p = node.getEnvironment().getParameters();

		//System.out.println((byte) p.getInt("pastry_rtBaseBitLength"));

		SOCKET_BUFFER_SIZE = p.getInt("pastry_socket_scm_socket_buffer_size");

		this.node = node;
		// We are only going to use one instance of this application on each
		// PastryNode
		this.endpoint = node.buildEndpoint(this, "PLMR-Server");

		// now we can receive messages
		this.endpoint.register();

		// open and register us as callback for a global socket for incoming req
		// from clients
		ServerSocketChannel channel = ServerSocketChannel.open();
		channel.configureBlocking(false);
		channel.socket().setReuseAddress(true);
		channel.socket().bind(new InetSocketAddress(plmrport));
		final plmr_srv_app psa = this;

		node.getEnvironment().getSelectorManager().register(
				channel,
				new SelectionKeyHandler() {
					public void accept(SelectionKey k) {
						try {
							new plmr_client_connection(k, psa);
						} catch (IOException ie) {
							System.out.println("exc in new_srv_app, accept");
							ie.printStackTrace();
						}
					}

					public void connect(SelectionKey key) {}

					public void read(SelectionKey key) {}

					public void write(SelectionKey key) {}
				},
				SelectionKey.OP_ACCEPT);

		//System.out.println("PLMR server listening on: " + plmrport);

		//and get lon lat from dlab
		//schedule as a proc, so that it doesnt burn cycles on the main thread
		((rice.pastry.PastryNode) node).process(new rice.Executable() {
			public Object execute() {
				try {
					String h = InetAddress.getLocalHost().getHostName();
					BufferedReader bfr = new BufferedReader(
							new InputStreamReader(new URL(DLAB_REQ_STRING + h)
									.openStream()));
					String line = bfr.readLine();
					if (line != null && !line.equals("")) {
						line = line.substring(1, line.length() - 1);
						String[] v = line.split(",");
						if (v.length > 0) {
							lon = v[0];
							lat = v[1];
						}
					}
				} catch (Exception ex) {
					System.out.println("exc in new_srv_app, get lon,lat");
					ex.printStackTrace();
				}
				return null;
			}
		}, new Continuation() {
			// will be called if success of the exec
			public void receiveResult(Object result) {}

			// will be called if failure
			public void receiveException(Exception result) {}
		});

		//open log file

		LogFile = new BufferedWriter(new FileWriter("fingertable.log"));

		((rice.pastry.commonapi.PastryEndpoint) endpoint)
				.scheduleMessageAtFixedRate(new LogFTMsg(), 100, 60000);

	}

	/**
	 * Called when we receive a message.
	 * 
	 * @param id
	 *            target ID
	 * @param message
	 *            message to us
	 */
	public void deliver(Id id, Message message) {
		if (!(message instanceof LogFTMsg))
			System.out.println(this + " received " + message + " for " + id);

		if (message == null)
			return;

		// see what tye of message it is
		//BuildTree Msg
		if (message instanceof BuildMessage) {

			//if we sent it, it means there are no more nodes in that subtree - drop
			if (((BuildMessage) message).from == node.getId())
				return;

			// see if we need to create query for it, and send down
			try {
				checkQuery((BuildMessage) message, null, true);
			} catch (Exception e) {
				e.printStackTrace();
			}
			//Send the result Msg

		} else if (message instanceof LogFTMsg) {
			((LogFTMsg)message).process(node, LogFile);
		} else if (message instanceof SendResMsg) {
			((SendResMsg)message).process(endpoint, node, perc_RingSize, lat, lon);
			this.queries.remove(((SendResMsg)message).q.query_ID);
			
			//a new result from a child
		} else if (message instanceof ResultMessage) {
			((ResultMessage)message).process(queries, endpoint, node);

			//a request for an rsync
		} else if (message instanceof RSyncReqMessage) {
			((RSyncReqMessage)message).process(node, endpoint, queries);
			//we got rsynced for one of our reqs
		} else if (message instanceof RSyncSentMessage) {
			((RSyncSentMessage)message).process(node, endpoint, queries, ssh_username);
		}
	}

	/**
	 * Called when you hear about a new neighbor.
	 * 
	 * @param handle
	 *            NodeHandle of node in question
	 * @param joined
	 *            state, true=joined, false=left
	 */
	public void update(NodeHandle handle, boolean joined) {
		rice.pastry.routing.RoutingTable rt = ((rice.pastry.PastryNode) node)
				.getRoutingTable();

		if (joined)
			perc_RingSize++;
		System.out.println(this + ": " + handle + " joined: " + joined
				+ ",now:" + perc_RingSize);
		System.out.println("#Fingers:" + rt.numEntries());

	}

	/**
	 * Called when a message travels along your path.
	 * 
	 * @param message
	 *            the Message
	 * @return true=forward, false drop
	 */
	public boolean forward(rice.p2p.commonapi.RouteMessage msg) {
		System.out.print(this + " forwarding:" + msg);
		if (msg != null)
			System.out.println(",to:" + msg.getDestinationId() + ",over:"
					+ msg.getNextHopHandle());

		//error handling for error when we are trying to forward an empty msg to someone else
		//than ourself
		if ((msg.getMessage()==null) && (msg.getDestinationId() != node.getId())){
			System.out.print(this + " dropping null msg to " + msg.getDestinationId());
			return false;
		}
		try {
			// only do something if in FP_FTT mode and res_msg
			if (msg instanceof ResultMessage) {
				ResultMessage rm = (ResultMessage) msg;

				Query q = (Query) queries.get(rm.query_ID);

				// if not known, just forward to next crossroad who hopefully knows
				// this q_id
				if ((q == null) || // not FTT - dont mess with it !
						(q.tree_type != BuildMessage.FP_FTT))
					return true;

				// it is FTT, calc. delay
				rm.path.delay2parent = (System.currentTimeMillis() - rm.path.TS_sent);

				// and add result
				q.addResult(rm.result);
				// also add pathInfo of that msg to q
				q.paths.add(rm.path);

				System.out.println("got a res for FP_FTT, saving and dropping");

				// check if aggregation should be scheduled
				checkAndAgg(q);
				// and drop msg
				return false;

				//if we overhear a buildMsg, just add the query to known
			} else if (msg instanceof BuildMessage) {
				System.out.println(this + " overheard BuildMsg:" + msg);
				// check if we know that query, signal no spreading down
				try {
					checkQuery((BuildMessage) msg, null, false);
				} catch (Exception e) {
					e.printStackTrace();
					return false;
				}
			}

		} catch (Exception ex) {
			ex.printStackTrace();
			return false;
		}

		return true;
	}

	/**
	 * returns a string description of this
	 * 
	 * @return the Description as a string
	 */
	public String toString() {
		return "PLMR-Server " + endpoint.getId();
	}

	/** 
	 * Private methods of this app 
	 */

	/**
	 * Takes a BuildMsg and does the following: 
	 * 	-makes Query Object and adds to known queries HT 
	 * 	-normalizes and/or creates Init/Agg-Execs if contained in msg 
	 * 	-schedules Init for execution 
	 * 	-schedules sending of Res depending on dist(root) 
	 * 	-and sends the msg down the tree It is called on every node until it hits leafs.
	 * 
	 * @param bm
	 *            the BuildMessage
	 * @param pcc
	 *            client_connx that we got this req from if we happen to be the
	 *            root
	 * @param sendDown
	 *            set to true if you want the message to be sent down the Tree
	 *            in case it is new
	 */
	private void checkQuery(BuildMessage bm, plmr_client_connection pcc,
			boolean sendDown) throws Exception {

		//make new query
		final Query q = new Query(
				bm.root,
				pcc,
				bm.from,
				bm.query_ID,
				bm.tree_type);

		q.checkQuery(bm, pcc, sendDown, queries, node, endpoint, ssh_username);
		
		}

	/**
	 * Called by checkQuery in case we need to spread the word down the tree
	 * 
	 * @param bm
	 *            the Incoming BuildMsg describing the tree
	 */
	private void sendBuildMsgDown(BuildMessage bm) {

		bm.sendDown(node, endpoint);
	}

	/** 
	 * external, public methods for things 
	 * */

	/**
	 * Called from client_conn to build a tree rooted at local node
	 * 
	 * @param bm
	 *            the Incoming BuildMsg describing the tree
	 * @param pcc
	 *            the Client Conn that the req came from for sending of the
	 *            results
	 */
	public void buildTreeHere(BuildMessage bm, plmr_client_connection pcc) {
		try {

			//			System.out.println("building a tree here for " + bm);

			//we are root so set all fields to us
			bm.root = node.getId();
			bm.to = bm.root;
			bm.senderProxyFor = bm.root;
			bm.from = bm.root;
			bm.timeout = 120000;

			bm.query_ID = InetAddress.getLocalHost().getHostName() + "/"
					+ System.currentTimeMillis();
			checkQuery(bm, pcc, true);

		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	/**
	 * Called to see if the query needs rescheduling of the Aggregator
	 * If it does the aggregator is scheduled for execution
	 * 
	 * @param q
	 *            The query in question
	 * @return int 
	 * 			Vague guess of time needed for completion, 0 iff only one
	 *         result (needs no processing)
	 */
	private int checkAndAgg(final Query q) {
		return q.checkAndAgg(node);
	}

	

}
